@@ -183,10 +183,28 @@ retry:
 	return 0;
 }
 
+static int validate_res_disk_paths(struct sanlk_resource *res)
+{
+	int i;
+
+	if (!res)
+		return -EINVAL;
+
+	for (i = 0; i < res->num_disks; i++) {
+		if (!memchr(res->disks[i].path, '\0', SANLK_PATH_LEN))
+			return -EINVAL;
+	}
+
+	return 0;
+}
+
 static int cmd_lockspace(int cmd, struct sanlk_lockspace *ls, uint32_t flags, uint32_t data)
 {
 	int rv, fd;
 
+	if (!ls || !ls->host_id_disk.path[0] || !memchr(ls->host_id_disk.path, '\0', SANLK_PATH_LEN))
+		return -EINVAL;
+
 	rv = connect_socket(&fd);
 	if (rv < 0)
 		return rv;
@@ -298,6 +316,51 @@ int sanlock_get_lockspaces(struct sanlk_lockspace **lss, int *lss_count,
 	return rv;
 }
 
+int sanlock_set_host(struct sanlk_lockspace *ls, uint32_t cmd, uint32_t flags, uint32_t io_timeout, void *host_data)
+{
+	int rv, fd;
+
+	if (!ls || !ls->name[0])
+		return -EINVAL;
+
+	/*
+	 * SANLK_SET_HOST_DEAD_EXT: host_data is sanlk_host
+	 */
+	if (cmd == SANLK_SET_HOST_DEAD_EXT) {
+		if (!host_data)
+			return -EINVAL;
+
+		rv = connect_socket(&fd);
+		if (rv < 0)
+			return rv;
+
+		rv = send_header(fd, SM_CMD_SET_HOST, flags,
+				 sizeof(struct sanlk_lockspace) + sizeof(struct sanlk_host),
+				 cmd, io_timeout);
+		if (rv < 0)
+			goto out;
+
+		rv = send_data(fd, ls, sizeof(struct sanlk_lockspace), 0);
+		if (rv < 0) {
+			rv = -errno;
+			goto out;
+		}
+
+		rv = send_data(fd, host_data, sizeof(struct sanlk_host), 0);
+		if (rv < 0) {
+			rv = -errno;
+			goto out;
+		}
+
+		rv = recv_result(fd);
+ out:
+		close(fd);
+		return rv;
+	}
+
+	return -EINVAL;
+}
+
 int sanlock_get_hosts(const char *ls_name, uint64_t host_id,
 		      struct sanlk_host **hss, int *hss_count,
 		      uint32_t flags)
@@ -477,7 +540,7 @@ int sanlock_read_lockspace_host(struct sanlk_lockspace *ls, uint32_t flags, uint
 	struct sm_header h;
 	int rv, fd;
 
-	if (!ls || !ls->host_id_disk.path[0])
+	if (!ls || !ls->host_id_disk.path[0] || !memchr(ls->host_id_disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -547,7 +610,7 @@ int sanlock_read_lockspace(struct sanlk_lockspace *ls, uint32_t flags, uint32_t
 	struct sm_header h;
 	int rv, fd;
 
-	if (!ls || !ls->host_id_disk.path[0])
+	if (!ls || !ls->host_id_disk.path[0] || !memchr(ls->host_id_disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -612,6 +675,10 @@ int sanlock_read_resource(struct sanlk_resource *res, uint32_t flags)
 	    !res->disks[0].path[0])
 		return -EINVAL;
 
+	rv = validate_res_disk_paths(res);
+	if (rv < 0)
+		return rv;
+
 	rv = connect_socket(&fd);
 	if (rv < 0)
 		return rv;
@@ -667,12 +734,63 @@ int sanlock_read_resource(struct sanlk_resource *res, uint32_t flags)
 	return rv;
 }
 
+int sanlock_init_lockspace_host(struct sanlk_lockspace *ls,
+                                char *our_host_name,
+                                uint64_t generation,
+                                uint64_t timestamp,
+                                uint32_t flags,
+                                uint32_t io_timeout)
+{
+	struct leader_record leader = { 0 };
+	char name[NAME_ID_SIZE + 1] = { 0 };
+	int rv, fd;
+
+	if (!ls || !ls->name[0] || !ls->host_id_disk.path[0] || !ls->host_id ||
+	    !memchr(ls->host_id_disk.path, '\0', SANLK_PATH_LEN))
+		return -EINVAL;
+
+	leader.owner_generation = generation;
+	leader.timestamp = timestamp;
+	leader.owner_id = ls->host_id;
+	if (our_host_name) {
+		strncpy(name, our_host_name, NAME_ID_SIZE);
+		memcpy(leader.resource_name, name, NAME_ID_SIZE);
+	}
+
+	rv = connect_socket(&fd);
+	if (rv < 0)
+		return rv;
+
+	rv = send_header(fd, SM_CMD_INIT_LOCKSPACE_HOST, flags,
+			 sizeof(struct sanlk_lockspace) + sizeof(struct leader_record),
+			 0, io_timeout);
+	if (rv < 0)
+		goto out;
+
+	rv = send_data(fd, ls, sizeof(struct sanlk_lockspace), 0);
+	if (rv < 0) {
+		rv = -errno;
+		goto out;
+	}
+
+	rv = send_data(fd, &leader, sizeof(struct leader_record), 0);
+	if (rv < 0) {
+		rv = -errno;
+		goto out;
+	}
+
+	rv = recv_result(fd);
+ out:
+	close(fd);
+	return rv;
+}
+
 int sanlock_write_lockspace(struct sanlk_lockspace *ls, int max_hosts,
 			    uint32_t flags, uint32_t io_timeout)
 {
 	int rv, fd;
 
-	if (!ls || !ls->host_id_disk.path[0])
+	if (!ls || !ls->host_id_disk.path[0] || !memchr(ls->host_id_disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -706,6 +824,10 @@ int sanlock_write_resource(struct sanlk_resource *res,
 	    !res->disks[0].path[0])
 		return -EINVAL;
 
+	rv = validate_res_disk_paths(res);
+	if (rv < 0)
+		return rv;
+
 	rv = connect_socket(&fd);
 	if (rv < 0)
 		return rv;
@@ -746,6 +868,10 @@ int sanlock_read_resource_owners(struct sanlk_resource *res, uint32_t flags,
 	    !res->disks[0].path[0])
 		return -EINVAL;
 
+	rv = validate_res_disk_paths(res);
+	if (rv < 0)
+		return rv;
+
 	rv = connect_socket(&fd);
 	if (rv < 0)
 		return rv;
@@ -1232,6 +1358,10 @@ int sanlock_acquire2(int sock, int pid, uint32_t flags,
 	if (res->num_disks != 1)
 		return -EINVAL;
 
+	rv = validate_res_disk_paths(res);
+	if (rv < 0)
+		return rv;
+
 	if (opt_in)
 		memcpy(&opt, opt_in, sizeof(struct sanlk_options));
 
@@ -1329,6 +1459,10 @@ int sanlock_acquire(int sock, int pid, uint32_t flags, int res_count,
 		if (res->num_disks > SANLK_MAX_DISKS)
 			return -EINVAL;
 
+		rv = validate_res_disk_paths(res);
+		if (rv < 0)
+			return rv;
+
 		datalen += (res->num_disks * sizeof(struct sanlk_disk));
 	}
 
@@ -1573,6 +1707,10 @@ int sanlock_request(uint32_t flags, uint32_t force_mode,
 	if (!res)
 		return -EINVAL;
 
+	rv = validate_res_disk_paths(res);
+	if (rv < 0)
+		return rv;
+
 	datalen = sizeof(struct sanlk_resource) +
 		  sizeof(struct sanlk_disk) * res->num_disks;
 
@@ -1616,6 +1754,8 @@ int sanlock_examine(uint32_t flags, struct sanlk_lockspace *ls,
 		return rv;
 
 	if (ls && ls->host_id_disk.path[0]) {
+		if (!memchr(ls->host_id_disk.path, '\0', SANLK_PATH_LEN))
+			return -EINVAL;
 		cmd = SM_CMD_EXAMINE_LOCKSPACE;
 		datalen = sizeof(struct sanlk_lockspace);
 		data = (char *)ls;
@@ -1749,7 +1889,7 @@ int sanlock_format_rindex(struct sanlk_rindex *rx, uint32_t flags)
 {
 	int rv, fd;
 
-	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0])
+	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !memchr(rx->disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -1777,7 +1917,7 @@ int sanlock_rebuild_rindex(struct sanlk_rindex *rx, uint32_t flags)
 {
 	int rv, fd;
 
-	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0])
+	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !memchr(rx->disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -1809,7 +1949,8 @@ int sanlock_update_rindex(struct sanlk_rindex *rx, uint32_t flags,
 
 	memset(&re_recv, 0, sizeof(re_recv));
 
-	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re)
+	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re ||
+	    !memchr(rx->disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -1865,7 +2006,8 @@ int sanlock_lookup_rindex(struct sanlk_rindex *rx, uint32_t flags,
 
 	memset(&re_recv, 0, sizeof(re_recv));
 
-	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re)
+	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re ||
+	    !memchr(rx->disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -1922,7 +2064,8 @@ int sanlock_create_resource(struct sanlk_rindex *rx, uint32_t flags,
 
 	memset(&re_recv, 0, sizeof(re_recv));
 
-	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re)
+	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re ||
+	    !memchr(rx->disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -1978,7 +2121,8 @@ int sanlock_delete_resource(struct sanlk_rindex *rx, uint32_t flags,
 
 	memset(&re_recv, 0, sizeof(re_recv));
 
-	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re)
+	if (!rx || !rx->lockspace_name[0] || !rx->disk.path[0] || !re ||
+	    !memchr(rx->disk.path, '\0', SANLK_PATH_LEN))
 		return -EINVAL;
 
 	rv = connect_socket(&fd);
@@ -2549,6 +2693,8 @@ const char *sanlock_strerror(int rv)
 		return "Lease is held by another host for shared acquire";
 	case SANLK_ACQUIRE_OWNED_RETRY:
 		return "Lease is owned by another host";
+	case SANLK_ACQUIRE_OWNED_NO_TIMEOUT:
+		return "Lease is owned by another notimeout host";
 	case SANLK_RELEASE_LVER:
 		return "Lease release version is incorrect";
 	case SANLK_RELEASE_OWNER:
