@@ -160,6 +160,7 @@ static const char *acquire_error_str(int error)
 	case SANLK_ACQUIRE_OWNED:
 	case SANLK_ACQUIRE_OTHER:
 	case SANLK_ACQUIRE_OWNED_RETRY:
+	case SANLK_ACQUIRE_OWNED_NO_TIMEOUT:
 		return "lease owned by other host";
 
 	case SANLK_ACQUIRE_SHRETRY:
@@ -342,10 +343,16 @@ static void cmd_acquire(struct task *task, struct cmd_args *ca, uint32_t cmd)
 			goto done;
 		}
 
-		/* zero out pad1 and pad2, see WARNING above */
 		for (j = 0; j < token->r.num_disks; j++) {
+			/* zero out pad1 and pad2, see WARNING above */
 			token->disks[j].sector_size = 0;
 			token->disks[j].fd = -1;
+
+			if (!token->disks[j].path[0] || !memchr(token->disks[j].path, '\0', SANLK_PATH_LEN)) {
+				free(token);
+				result = -ENODEV;
+				goto done;
+			}
 		}
 
 		token->token_id = token_id_counter++;
@@ -435,6 +442,7 @@ static void cmd_acquire(struct task *task, struct cmd_args *ca, uint32_t cmd)
 			case SANLK_ACQUIRE_OWNED:
 			case SANLK_ACQUIRE_OTHER:
 			case SANLK_ACQUIRE_OWNED_RETRY:
+			case SANLK_ACQUIRE_OWNED_NO_TIMEOUT:
 				if ((cmd == SM_CMD_ACQUIRE2) &&
 				    (owner_host.host_id > 0) &&
 				    (owner_host.host_id <= DEFAULT_MAX_HOSTS))
@@ -512,8 +520,8 @@ static void cmd_acquire(struct task *task, struct cmd_args *ca, uint32_t cmd)
  done:
 	pthread_mutex_lock(&spaces_mutex);
 	pthread_mutex_lock(&cl->mutex);
-	log_cmd(cmd, "cmd_acquire %d,%d,%d result %d pid_dead %d",
-		  cl_ci, cl_fd, cl_pid, result, cl->pid_dead);
+	log_cmd(cmd, "cmd_acquire %d,%d,%d result %d pid_dead %d owner_host_send %d",
+		  cl_ci, cl_fd, cl_pid, result, cl->pid_dead, owner_host_send);
 
 	pid_dead = cl->pid_dead;
 	cl->cmd_active = 0;
@@ -558,7 +566,7 @@ static void cmd_acquire(struct task *task, struct cmd_args *ca, uint32_t cmd)
 		sp = find_lockspace(res.lockspace_name);
 		if (sp) {
 			hs = &sp->host_status[owner_host.host_id-1];
-			owner_host.flags = get_host_flag(sp, hs);
+			owner_host.flags = get_host_flag(sp->host_id, hs);
 			memcpy(owner_host_name, hs->owner_name, NAME_ID_SIZE);
 		}
 	}
@@ -1092,10 +1100,15 @@ static void cmd_request(struct task *task, struct cmd_args *ca, uint32_t cmd)
 		goto reply_free;
 	}
 
-	/* zero out pad1 and pad2, see WARNING above */
 	for (j = 0; j < token->r.num_disks; j++) {
+		/* zero out pad1 and pad2, see WARNING above */
 		token->disks[j].sector_size = 0;
 		token->disks[j].fd = -1;
+
+		if (!token->disks[j].path[0] || !memchr(token->disks[j].path, '\0', SANLK_PATH_LEN)) {
+			result = -ENODEV;
+			goto reply_free;
+		}
 	}
 
 	log_cmd(cmd, "cmd_request %d,%d force_mode %u %.48s:%.48s:%.256s:%llu",
@@ -1187,6 +1200,48 @@ static void cmd_examine(struct task *task GNUC_UNUSED, struct cmd_args *ca, uint
 	client_resume(ca->ci_in);
 }
 
+static void cmd_set_host(struct task *task, struct cmd_args *ca, uint32_t cmd)
+{
+	struct sanlk_lockspace lockspace;
+	struct sanlk_host host;
+	int fd, rv, result;
+
+	fd = client[ca->ci_in].fd;
+
+	rv = recv_loop(fd, &lockspace, sizeof(struct sanlk_lockspace), MSG_WAITALL);
+	if (rv != sizeof(struct sanlk_lockspace)) {
+		log_error("cmd_set_host %d,%d recv ls %d %d", ca->ci_in, fd, rv, errno);
+		result = -ENOTCONN;
+		goto reply;
+	}
+
+	rv = recv_loop(fd, &host, sizeof(struct sanlk_host), MSG_WAITALL);
+	if (rv != sizeof(struct sanlk_host)) {
+		log_error("cmd_set_host %d,%d recv host %d %d", ca->ci_in, fd, rv, errno);
+		result = -ENOTCONN;
+		goto reply;
+	}
+
+	if (lockspace.host_id_disk.path[0] && !memchr(lockspace.host_id_disk.path, '\0', SANLK_PATH_LEN)) {
+		log_error("Invalid lockspace path.");
+		result = -ENODEV;
+		goto reply;
+	}
+
+	log_cmd(cmd, "cmd_set_host %d,%d %.48s host %llu %llu op %u", ca->ci_in, fd, lockspace.name,
+		(unsigned long long)host.host_id, (unsigned long long)host.generation, ca->header.data);
+
+	if (ca->header.data == SANLK_SET_HOST_DEAD_EXT)
+		result = lockspace_set_host_dead_ext(task, &lockspace, &host, ca->header.data2);
+	else
+		result = -EINVAL;
+
+	log_cmd(cmd, "cmd_set_host %d,%d result %d", ca->ci_in, fd, result);
+ reply:
+	send_result(ca->ci_in, fd, &ca->header, result);
+	client_resume(ca->ci_in);
+}
+
 static void cmd_set_lvb(struct task *task GNUC_UNUSED, struct cmd_args *ca, uint32_t cmd)
 {
 	struct sanlk_resource res;
@@ -1347,6 +1402,15 @@ void daemon_shutdown_reply(void)
 	client_resume(shutdown_reply_ci);
 }
 
+static int validate_lockspace_path(struct sanlk_lockspace *ls)
+{
+	if (!ls->host_id_disk.path[0] || !memchr(ls->host_id_disk.path, '\0', SANLK_PATH_LEN)) {
+		log_error("Invalid lockspace path.");
+		return -ENODEV;
+	}
+	return 0;
+}
+
 static void cmd_add_lockspace(struct cmd_args *ca, uint32_t cmd)
 {
 	struct sanlk_lockspace lockspace;
@@ -1366,12 +1430,20 @@ static void cmd_add_lockspace(struct cmd_args *ca, uint32_t cmd)
 		goto reply;
 	}
 
-	log_cmd(cmd, "cmd_add_lockspace %d,%d %.48s:%llu:%s:%llu flags %x timeout %u",
+	rv = validate_lockspace_path(&lockspace);
+	if (rv < 0) {
+		result = rv;
+		goto reply;
+	}
+
+	log_cmd(cmd, "cmd_add_lockspace %d,%d %.48s:%llu:%s:%llu ls_flags 0x%x cmd_flags 0x%x timeout %u",
 		  ca->ci_in, fd, lockspace.name,
 		  (unsigned long long)lockspace.host_id,
 		  lockspace.host_id_disk.path,
 		  (unsigned long long)lockspace.host_id_disk.offset,
-		  ca->header.cmd_flags, ca->header.data);
+		  lockspace.flags,
+		  ca->header.cmd_flags,
+		  ca->header.data);
 
 	io_timeout = ca->header.data;
 	if (!io_timeout)
@@ -1415,7 +1487,13 @@ static void cmd_inq_lockspace(struct cmd_args *ca, uint32_t cmd)
 		goto reply;
 	}
 
-	log_cmd(cmd, "cmd_inq_lockspace %d,%d %.48s:%llu:%s:%llu flags %x",
+	rv = validate_lockspace_path(&lockspace);
+	if (rv < 0) {
+		result = rv;
+		goto reply;
+	}
+
+	log_cmd(cmd, "cmd_inq_lockspace %d,%d %.48s:%llu:%s:%llu cmd_flags %x",
 		  ca->ci_in, fd, lockspace.name,
 		  (unsigned long long)lockspace.host_id,
 		  lockspace.host_id_disk.path,
@@ -1499,8 +1577,14 @@ static void cmd_rem_lockspace(struct cmd_args *ca, uint32_t cmd)
 		goto reply;
 	}
 
-	log_cmd(cmd, "cmd_rem_lockspace %d,%d %.48s flags %x",
-		  ca->ci_in, fd, lockspace.name, ca->header.cmd_flags);
+	rv = validate_lockspace_path(&lockspace);
+	if (rv < 0) {
+		result = rv;
+		goto reply;
+	}
+
+	log_cmd(cmd, "cmd_rem_lockspace %d,%d %.48s cmd_flags %x",
+		ca->ci_in, fd, lockspace.name, ca->header.cmd_flags);
 
 	if (ca->header.cmd_flags & SANLK_REM_UNUSED) {
 		if (lockspace_is_used(&lockspace)) {
@@ -1549,7 +1633,7 @@ static void cmd_align(struct task *task GNUC_UNUSED, struct cmd_args *ca, uint32
 
 	log_cmd(cmd, "cmd_align %d,%d", ca->ci_in, fd);
 
-	if (!disk.path[0]) {
+	if (!disk.path[0] || !memchr(disk.path, '\0', SANLK_PATH_LEN)) {
 		result = -ENODEV;
 		goto reply;
 	}
@@ -1596,6 +1680,12 @@ static void cmd_read_lockspace(struct task *task, struct cmd_args *ca, uint32_t
 		goto reply;
 	}
 
+	rv = validate_lockspace_path(&lockspace);
+	if (rv < 0) {
+		result = rv;
+		goto reply;
+	}
+
 	if (!lockspace.host_id)
 		host_id = 1;
 	else
@@ -1608,11 +1698,6 @@ static void cmd_read_lockspace(struct task *task, struct cmd_args *ca, uint32_t
 		  lockspace.host_id_disk.path,
 		  (unsigned long long)lockspace.host_id_disk.offset);
 
-	if (!lockspace.host_id_disk.path[0]) {
-		result = -ENODEV;
-		goto reply;
-	}
-
 	memset(&sd, 0, sizeof(struct sync_disk));
 	memcpy(&sd, &lockspace.host_id_disk, sizeof(struct sanlk_disk));
 	sd.fd = -1;
@@ -1627,20 +1712,28 @@ static void cmd_read_lockspace(struct task *task, struct cmd_args *ca, uint32_t
 	align_size = sanlk_lsf_align_flag_to_size(lockspace.flags);
 
 	if (!sector_size) {
-		/* reads the first leader record to get sector size */
-		result = delta_read_lockspace_sizes(task, &sd, com.io_timeout, &sector_size, &align_size);
-		if (result < 0)
+		result = delta_read_lockspace_sizes(task, &sd, host_id, com.io_timeout, &sector_size, &align_size);
+		if (result < 0) {
+			log_cmd(cmd, "cmd_read_lockspace%s %d,%d host_id %llu delta_read_lockspace_sizes error %d",
+				with_host ? "_host" : "", ca->ci_in, fd, (unsigned long long)host_id, result);
 			goto out_close;
+		}
 		if ((sector_size != 512) && (sector_size != 4096)) {
+			log_cmd(cmd, "cmd_read_lockspace%s %d,%d host_id %llu invalid sector size %d",
+				with_host ? "_host" : "", ca->ci_in, fd, (unsigned long long)host_id, sector_size);
 			result = -EINVAL;
 			goto out_close;
 		}
 	}
 
-	result = delta_read_lockspace(task, &sd, sector_size, align_size, host_id, &lockspace,
-				      com.io_timeout, &host);
-	if (result == SANLK_OK)
+	result = delta_read_lockspace(task, &sd, sector_size, host_id, &lockspace, com.io_timeout, &host);
+	if (result == SANLK_OK) {
 		result = 0;
+	} else {
+		log_cmd(cmd, "cmd_read_lockspace%s %d,%d host_id %llu delta_read_lockspace error %d",
+			with_host ? "_host" : "", ca->ci_in, fd, (unsigned long long)host_id, result);
+	}
+
 
  out_close:
 	close_disks(&sd, 1);
@@ -1721,10 +1814,15 @@ static void cmd_read_resource(struct task *task, struct cmd_args *ca, uint32_t c
 		goto reply;
 	}
 
-	/* zero out pad1 and pad2, see WARNING above */
 	for (j = 0; j < token->r.num_disks; j++) {
+		/* zero out pad1 and pad2, see WARNING above */
 		token->disks[j].sector_size = 0;
 		token->disks[j].fd = -1;
+
+		if (!token->disks[j].path[0] || !memchr(token->disks[j].path, '\0', SANLK_PATH_LEN)) {
+			result = -ENODEV;
+			goto reply;
+		}
 	}
 
 	log_cmd(cmd, "cmd_read_resource %d,%d %.256s:%llu",
@@ -1822,10 +1920,15 @@ static void cmd_read_resource_owners(struct task *task, struct cmd_args *ca, uin
 		goto reply;
 	}
 
-	/* zero out pad1 and pad2, see WARNING above */
 	for (j = 0; j < token->r.num_disks; j++) {
+		/* zero out pad1 and pad2, see WARNING above */
 		token->disks[j].sector_size = 0;
 		token->disks[j].fd = -1;
+
+		if (!token->disks[j].path[0] || !memchr(token->disks[j].path, '\0', SANLK_PATH_LEN)) {
+			result = -ENODEV;
+			goto reply;
+		}
 	}
 
 	log_cmd(cmd, "cmd_read_resource_owners %d,%d %.256s:%llu",
@@ -1876,6 +1979,81 @@ static void cmd_read_resource_owners(struct task *task, struct cmd_args *ca, uin
 	client_resume(ca->ci_in);
 }
 
+static void cmd_init_lockspace_host(struct task *task, struct cmd_args *ca, uint32_t cmd)
+{
+	struct sanlk_lockspace lockspace;
+	struct leader_record leader;
+	struct sync_disk sd;
+	int fd, rv, result;
+	int io_timeout = com.io_timeout;
+
+	fd = client[ca->ci_in].fd;
+
+	rv = recv_loop(fd, &lockspace, sizeof(struct sanlk_lockspace), MSG_WAITALL);
+	if (rv != sizeof(struct sanlk_lockspace)) {
+		log_error("cmd_init_lockspace_host %d,%d recv lockspace %d %d",
+			   ca->ci_in, fd, rv, errno);
+		result = -ENOTCONN;
+		goto reply;
+	}
+
+	rv = recv_loop(fd, &leader, sizeof(struct leader_record), MSG_WAITALL);
+	if (rv != sizeof(struct leader_record)) {
+		log_error("cmd_init_lockspace_host %d,%d recv leader %d %d",
+			   ca->ci_in, fd, rv, errno);
+		result = -ENOTCONN;
+		goto reply;
+	}
+
+	rv = validate_lockspace_path(&lockspace);
+	if (rv < 0) {
+		result = rv;
+		goto reply;
+	}
+
+	log_cmd(cmd, "cmd_init_lockspace_host %d,%d %.48s:%llu:%s:%llu ls_flags 0x%x cmd_flags 0x%x",
+		  ca->ci_in, fd, lockspace.name,
+		  (unsigned long long)lockspace.host_id,
+		  lockspace.host_id_disk.path,
+		  (unsigned long long)lockspace.host_id_disk.offset,
+		  lockspace.flags,
+		  ca->header.cmd_flags);
+
+	if (!lockspace.host_id) {
+		result = -EINVAL;
+		goto reply;
+	}
+
+	memset(&sd, 0, sizeof(struct sync_disk));
+	memcpy(&sd, &lockspace.host_id_disk, sizeof(struct sanlk_disk));
+	sd.fd = -1;
+
+	rv = open_disk(&sd);
+	if (rv < 0) {
+		result = -ENODEV;
+		goto reply;
+	}
+
+	if (ca->header.data2)
+		io_timeout = ca->header.data2;
+
+	result = delta_lease_init_host(task, &lockspace,
+				       leader.resource_name[0] ? leader.resource_name : NULL,
+				       leader.owner_generation,
+				       leader.timestamp,
+				       io_timeout,
+				       &sd);
+	if (result == SANLK_OK)
+		result = 0;
+
+	close_disks(&sd, 1);
+ reply:
+	log_cmd(cmd, "cmd_init_lockspace_host %d,%d done %d", ca->ci_in, fd, result);
+
+	send_result(ca->ci_in, fd, &ca->header, result);
+	client_resume(ca->ci_in);
+}
+
 static void cmd_write_lockspace(struct task *task, struct cmd_args *ca, uint32_t cmd)
 {
 	struct sanlk_lockspace lockspace;
@@ -1893,17 +2071,19 @@ static void cmd_write_lockspace(struct task *task, struct cmd_args *ca, uint32_t
 		goto reply;
 	}
 
-	log_cmd(cmd, "cmd_write_lockspace %d,%d %.48s:%llu:%s:%llu 0x%x",
+	rv = validate_lockspace_path(&lockspace);
+	if (rv < 0) {
+		result = rv;
+		goto reply;
+	}
+
+	log_cmd(cmd, "cmd_write_lockspace %d,%d %.48s:%llu:%s:%llu ls_flags 0x%x cmd_flags 0x%x",
 		  ca->ci_in, fd, lockspace.name,
 		  (unsigned long long)lockspace.host_id,
 		  lockspace.host_id_disk.path,
 		  (unsigned long long)lockspace.host_id_disk.offset,
-		  lockspace.flags);
-
-	if (!lockspace.host_id_disk.path[0]) {
-		result = -ENODEV;
-		goto reply;
-	}
+		  lockspace.flags,
+		  ca->header.cmd_flags);
 
 	/* No longer used, max_hosts is derived from sector/align sizes. */
 	/* max_hosts = ca->header.data; */
@@ -1921,7 +2101,15 @@ static void cmd_write_lockspace(struct task *task, struct cmd_args *ca, uint32_t
 	if (ca->header.data2)
 		io_timeout = ca->header.data2;
 
-	result = delta_lease_init(task, &lockspace, io_timeout, &sd);
+	if (!ca->header.cmd_flags)
+		result = delta_lease_init(task, &lockspace, io_timeout, &sd);
+	else if (ca->header.cmd_flags & SANLK_WRITE_LS_FLAGS)
+		result = delta_lease_update_lockspace(task, &lockspace, io_timeout, &sd);
+	else {
+		log_error("cmd_write_lockspace %d,%d unknown flags 0x%x",
+			  ca->ci_in, fd, ca->header.cmd_flags);
+		result = -EINVAL;
+	}
 
 	close_disks(&sd, 1);
  reply:
@@ -1986,13 +2174,18 @@ static void cmd_write_resource(struct task *task, struct cmd_args *ca, uint32_t
 		goto reply;
 	}
 
-	/* zero out pad1 and pad2, see WARNING above */
 	for (j = 0; j < token->r.num_disks; j++) {
+		/* zero out pad1 and pad2, see WARNING above */
 		token->disks[j].sector_size = 0;
 		token->disks[j].fd = -1;
+
+		if (!token->disks[j].path[0] || !memchr(token->disks[j].path, '\0', SANLK_PATH_LEN)) {
+			result = -ENODEV;
+			goto reply;
+		}
 	}
 
-	log_cmd(cmd, "cmd_write_resource %d,%d %.48s:%.48s:%.256s:%llu 0x%x",
+	log_cmd(cmd, "cmd_write_resource %d,%d %.48s:%.48s:%.256s:%llu res_flags 0x%x",
 		  ca->ci_in, fd,
 		  token->r.lockspace_name,
 		  token->r.name,
@@ -2040,7 +2233,7 @@ static void cmd_killpath(struct task *task, struct cmd_args *ca, uint32_t cmd)
 
 	cl = &client[cl_ci];
 
-	log_cmd(cmd, "cmd_killpath %d,%d,%d flags %x",
+	log_cmd(cmd, "cmd_killpath %d,%d,%d cmd_flags 0x%x",
 		  cl_ci, cl_fd, cl_pid, ca->header.cmd_flags);
 
 	rv = recv_loop(cl_fd, cl->killpath, SANLK_HELPER_PATH_LEN, MSG_WAITALL);
@@ -2266,6 +2459,9 @@ void call_cmd_thread(struct task *task, struct cmd_args *ca)
 	case SM_CMD_ALIGN:
 		cmd_align(task, ca, cmd);
 		break;
+	case SM_CMD_INIT_LOCKSPACE_HOST:
+		cmd_init_lockspace_host(task, ca, cmd);
+		break;
 	case SM_CMD_WRITE_LOCKSPACE:
 		cmd_write_lockspace(task, ca, cmd);
 		break;
@@ -2289,6 +2485,9 @@ void call_cmd_thread(struct task *task, struct cmd_args *ca)
 	case SM_CMD_KILLPATH:
 		cmd_killpath(task, ca, cmd);
 		break;
+	case SM_CMD_SET_HOST:
+		cmd_set_host(task, ca, cmd);
+		break;
 	case SM_CMD_SET_LVB:
 		cmd_set_lvb(task, ca, cmd);
 		break;
@@ -2379,7 +2578,8 @@ static int print_state_daemon(char *str)
 		 "version_str=%s "
 		 "version_num=%u.%u.%u "
 		 "version_hex=%08x "
-		 "smproto_hex=%08x",
+		 "smproto_hex=%08x "
+		 "create_delta_34=%d",
 		 our_host_name_global,
 		 com.use_watchdog,
 		 com.high_priority,
@@ -2414,7 +2614,8 @@ static int print_state_daemon(char *str)
 		 sanlock_version_minor,
 		 sanlock_version_patch,
 		 sanlock_version_combined,
-		 SM_PROTO);
+		 SM_PROTO,
+		 com.create_delta_34);
 
 	return strlen(str) + 1;
 }
@@ -2459,6 +2660,7 @@ static int print_state_lockspace(struct space *sp, char *str, const char *list_n
 	snprintf(str, SANLK_STATE_MAXSTR-1,
 		 "list=%s "
 		 "space_id=%u "
+		 "version=0x%08x "
 		 "io_timeout=%d "
 		 "sector_size=%d "
 		 "align_size=%d "
@@ -2469,6 +2671,7 @@ static int print_state_lockspace(struct space *sp, char *str, const char *list_n
 		 "used_retries=%u "
 		 "external_used=%d "
 		 "used_by_orphans=%d "
+		 "no_timeout=%d "
 		 "renewal_read_extend_sec=%u "
 		 "set_max_sectors_kb=%u "
 		 "corrupt_result=%d "
@@ -2480,6 +2683,7 @@ static int print_state_lockspace(struct space *sp, char *str, const char *list_n
 		 "renewal_last_success=%llu",
 		 list_name,
 		 sp->space_id,
+		 sp->version,
 		 sp->io_timeout,
 		 sp->sector_size,
 		 sp->align_size,
@@ -2490,6 +2694,7 @@ static int print_state_lockspace(struct space *sp, char *str, const char *list_n
 		 sp->used_retries,
 		 (sp->flags & SP_EXTERNAL_USED) ? 1 : 0,
 		 (sp->flags & SP_USED_BY_ORPHANS) ? 1 : 0,
+		 sp->no_timeout,
 		 sp->renewal_read_extend_sec,
 		 sp->set_max_sectors_kb,
 		 sp->lease_status.corrupt_result,
@@ -2533,8 +2738,12 @@ static int print_state_resource(struct resource *r, char *str, const char *list_
 
 static int print_state_host(struct host_status *hs, char *str)
 {
+	uint32_t flag;
+
 	memset(str, 0, SANLK_STATE_MAXSTR);
 
+	flag = get_host_flag(0, hs);
+
 	snprintf(str, SANLK_STATE_MAXSTR-1,
 		 "last_check=%llu "
 		 "last_live=%llu "
@@ -2543,6 +2752,12 @@ static int print_state_host(struct host_status *hs, char *str)
 		 "owner_generation=%llu "
 		 "timestamp=%llu "
 		 "io_timeout=%u "
+		 "no_timeout=%u "
+		 "dead_ext=%u "
+		 "found_dead_ext_gen=%llu "
+		 "wrote_dead_ext_gen=%llu "
+		 "host_flags=0x%x "
+		 "host_state=%s "
 		 "owner_name=%.48s",
 		 (unsigned long long)hs->last_check,
 		 (unsigned long long)hs->last_live,
@@ -2551,6 +2766,12 @@ static int print_state_host(struct host_status *hs, char *str)
 		 (unsigned long long)hs->owner_generation,
 		 (unsigned long long)hs->timestamp,
 		 hs->io_timeout,
+		 hs->no_timeout,
+		 hs->dead_ext,
+		 (unsigned long long)hs->found_dead_ext_gen,
+		 (unsigned long long)hs->wrote_dead_ext_gen,
+		 flag,
+		 host_state_str(flag),
 		 hs->owner_name);
 
 	return strlen(str) + 1;
