@@ -25,6 +25,7 @@
 
 #include "sanlock_internal.h"
 #include "sanlock.h"
+#include "sanlock_admin.h"
 #include "diskio.h"
 #include "ondisk.h"
 #include "direct.h"
@@ -96,10 +97,12 @@ static int verify_leader(struct sync_disk *disk,
 		goto fail;
 	}
 
-	if ((lr->version & 0xFFFF0000) != DELTA_DISK_VERSION_MAJOR) {
-		log_error("verify_leader %llu wrong version %x %s",
+	if ((lr->version & 0xFFFF0000) > DELTA_DISK_VERSION_MAJOR) {
+		log_error("verify_leader %llu newer major version leader 0x%x local 0x%x %s",
 			  (unsigned long long)host_id,
-			  lr->version, disk->path);
+			  lr->version,
+			  DELTA_DISK_VERSION_MAJOR|DELTA_DISK_VERSION_MINOR,
+			  disk->path);
 		result = SANLK_LEADER_VERSION;
 		goto fail;
 	}
@@ -144,13 +147,9 @@ static int verify_leader(struct sync_disk *disk,
 	return result;
 }
 
-
-/* read the lockspace name and io_timeout given the disk location */
-
 int delta_read_lockspace(struct task *task,
 			 struct sync_disk *disk,
-			 int sector_size_hint,
-			 int align_size_hint,
+			 int sector_size,
 			 uint64_t host_id,
 			 struct sanlk_lockspace *ls,
 			 int io_timeout,
@@ -167,12 +166,7 @@ int delta_read_lockspace(struct task *task,
 
 	memset(&leader_end, 0, sizeof(struct leader_record));
 
-	/*
-	 * All we need to read is the leader_record which is returned whether
-	 * or not the sector_size_hint is wrong or not.
-	 */
-
-	rv = read_sectors(disk, sector_size_hint, host_id - 1, 1, (char *)&leader_end, sizeof(struct leader_record),
+	rv = read_sectors(disk, sector_size, host_id - 1, 1, (char *)&leader_end, sizeof(struct leader_record),
 			  task, io_timeout, "read_lockspace");
 	if (rv < 0)
 		return rv;
@@ -204,11 +198,16 @@ int delta_read_lockspace(struct task *task,
 		ls->flags |= sanlk_lsf_sector_size_to_flag(leader.sector_size);
 		ls->flags |= sanlk_lsf_align_size_to_flag(align_size);
 
+		if (leader.flags & LFL_NO_TIMEOUT)
+			ls->flags |= SANLK_LSF_NO_TIMEOUT;
+
 		if (hs) {
 			hs->host_id = leader.owner_id;
 			hs->generation = leader.owner_generation;
 			hs->timestamp = leader.timestamp;
 			hs->io_timeout = leader.io_timeout;
+			if (leader.flags & LFL_DEAD_EXT)
+				hs->flags |= SANLK_HOST_DEAD_EXT;
 		}
 	}
 
@@ -217,6 +216,7 @@ int delta_read_lockspace(struct task *task,
 
 int delta_read_lockspace_sizes(struct task *task,
 			 struct sync_disk *disk,
+			 uint64_t host_id,
 			 int io_timeout,
 			 int *sector_size,
 			 int *align_size)
@@ -227,13 +227,39 @@ int delta_read_lockspace_sizes(struct task *task,
 
 	memset(&leader_end, 0, sizeof(struct leader_record));
 
-	/*
-	 * read the first 4k, which either includes one 4k delta lease or 8 512b
-	 * delta leases.  In either case, we only look at the initial leader
-	 * record to get to the sector size.
-	 */
+	/* With no host_id, get sizes from any leader record. */
+	if (!host_id)
+		host_id = 1;
 
-	rv = read_sectors(disk, 4096, 0, 1, (char *)&leader_end, sizeof(struct leader_record),
+	rv = read_sectors(disk, 512, host_id - 1, 1, (char *)&leader_end, sizeof(struct leader_record),
+			  task, io_timeout, "read_lockspace_sector_size");
+	if (rv < 0)
+		goto next;
+
+	leader_record_in(&leader_end, &leader);
+
+	if (leader.magic != DELTA_DISK_MAGIC)
+		goto next;
+
+	if ((leader.version & 0xFFFF0000) > DELTA_DISK_VERSION_MAJOR)
+		goto next;
+
+	if (leader.owner_id && (leader.owner_id != host_id))
+		goto next;
+
+	if (leader.sector_size != 512)
+		return SANLK_LEADER_SECTORSIZE;
+
+	*sector_size = leader.sector_size;
+
+	*align_size = leader_align_size_from_flag(leader.flags);
+	if (!*align_size)
+		*align_size = sector_size_to_align_size_old(leader.sector_size);
+
+	return SANLK_OK;
+
+ next:
+	rv = read_sectors(disk, 4096, host_id - 1, 1, (char *)&leader_end, sizeof(struct leader_record),
 			  task, io_timeout, "read_lockspace_sector_size");
 	if (rv < 0)
 		return rv;
@@ -243,9 +269,15 @@ int delta_read_lockspace_sizes(struct task *task,
 	if (leader.magic != DELTA_DISK_MAGIC)
 		return SANLK_LEADER_MAGIC;
 
-	if ((leader.version & 0xFFFF0000) != DELTA_DISK_VERSION_MAJOR)
+	if ((leader.version & 0xFFFF0000) > DELTA_DISK_VERSION_MAJOR)
 		return SANLK_LEADER_VERSION;
 
+	if (leader.owner_id && (leader.owner_id != host_id))
+		return SANLK_LEADER_DIFF;
+
+	if (leader.sector_size != 4096)
+		return SANLK_LEADER_SECTORSIZE;
+
 	*sector_size = leader.sector_size;
 
 	*align_size = leader_align_size_from_flag(leader.flags);
@@ -322,6 +354,58 @@ int delta_lease_leader_clobber(struct task *task, int io_timeout,
 	return SANLK_OK;
 }
 
+static void make_local_copy_path(char *path, int len, struct space *sp)
+{
+	snprintf(path, len, "/var/lib/sanlock/delta_lease_%s_%llu",
+		 sp->space_name, (unsigned long long)sp->host_id);
+}
+
+static void delta_local_copy_read(struct space *sp, uint64_t *prev_gen)
+{
+	char path[PATH_MAX];
+	FILE *file;
+	int rv;
+
+	*prev_gen = 0;
+
+	make_local_copy_path(path, sizeof(path), sp);
+
+	file = fopen(path, "r");
+	if (!file) {
+		log_space(sp, "delta_local_copy_read open error %d %s", errno, path);
+		return;
+	}
+
+	rv = fscanf(file, "%llu", (unsigned long long *)prev_gen);
+	if (rv != 1)
+		log_space(sp, "delta_local_copy_read read error %d %s", errno, path);
+
+	fclose(file);
+}
+
+static void delta_local_copy_write(struct space *sp, uint64_t next_gen)
+{
+	char path[PATH_MAX];
+	FILE *file;
+	int rv;
+
+	make_local_copy_path(path, sizeof(path), sp);
+
+	file = fopen(path, "w");
+	if (!file) {
+		log_space(sp, "delta_local_copy_write open error %d", errno);
+		return;
+	}
+
+	rv = fprintf(file, "%llu\n", (unsigned long long)next_gen);
+	if (rv < 0)
+		log_erros(sp, "delta_local_copy_write write error %d %s", errno, path);
+	if (fflush(file))
+		log_erros(sp, "delta_local_copy_write flush error %d %s", errno, path);
+	if (fclose(file))
+		log_erros(sp, "delta_local_copy_write close error %d %s", errno, path);
+}
+
 /*
  * delta_lease_acquire:
  * set the owner of host_id to our_host_name.
@@ -350,7 +434,10 @@ int delta_lease_acquire(struct task *task,
 	struct leader_record leader1;
 	struct leader_record leader_end;
 	uint64_t new_ts;
+	uint64_t prev_generation;
+	uint64_t next_generation;
 	uint32_t checksum;
+	uint32_t version;
 	int other_io_timeout, other_host_dead_seconds, other_id_renewal_seconds;
 	int i, error, rv, delay, delta_large_delay;
 	int is_free, is_same;
@@ -359,6 +446,15 @@ int delta_lease_acquire(struct task *task,
 	log_space(sp, "delta_acquire begin %.48s:%llu",
 		  sp->space_name, (unsigned long long)host_id);
 
+	/*
+	 * Keep a local copy of the last started generation number
+	 * in case the delta lease on disk is reverted by another
+	 * host writing the DEAD_EXT flag.  The local copy of the
+	 * generation number ensures we don't reuse the same
+	 * generation number that we started last.
+	 */
+	delta_local_copy_read(sp, &prev_generation);
+
 	error = delta_lease_leader_read(task, sp->sector_size, sp->io_timeout, disk, space_name, host_id, &leader,
 					"delta_acquire_begin");
 	if (error < 0) {
@@ -366,6 +462,8 @@ int delta_lease_acquire(struct task *task,
 		return error;
 	}
 
+	version = leader.version;
+
 	other_io_timeout = leader.io_timeout;
 	if (!other_io_timeout)
 		other_io_timeout = sp->io_timeout;
@@ -492,12 +590,72 @@ int delta_lease_acquire(struct task *task,
 	}
 
  write_new:
+
+	if (leader.owner_generation && (prev_generation != leader.owner_generation)) {
+		log_erros(sp, "delta_acquire %llu wrong prev_generation %llu leader %llu %llu %llu %s",
+			  (unsigned long long)host_id,
+			  (unsigned long long)prev_generation,
+			  (unsigned long long)leader.owner_id,
+			  (unsigned long long)leader.owner_generation,
+			  (unsigned long long)leader.timestamp,
+			  (leader.flags & LFL_DEAD_EXT) ? "dead_ext" : "");
+	}
+
+	if (leader.flags & LFL_DEAD_EXT) {
+		leader.flags &= ~LFL_DEAD_EXT;
+
+		if (!prev_generation) {
+			/*
+			 * This shouldn't happen. If it does happen,
+			 * some extra caution could be to do
+			 * next = leader.gen + 2, but skipping a gen
+			 * unnecessarily could also complicate external
+			 * assumptions about the normal gen sequence.
+			 */
+			next_generation = leader.owner_generation + 1;
+
+			log_space(sp, "delta_acquire %llu leader dead_ext gen %llu no prev_generation next %llu",
+				  (unsigned long long)host_id,
+				  (unsigned long long)leader.owner_generation,
+				  (unsigned long long)next_generation);
+		} else if (leader.owner_generation + 1 == prev_generation) {
+			/*
+			 * our last generation was clobbered by a DEAD_EXT
+			 * write from another host, and then we failed before
+			 * renewing and replacing that late DEAD_EXT write.
+			 * We must +2 to avoid reusing the previous gen.
+			 */
+			next_generation = leader.owner_generation + 2;
+
+			log_space(sp, "delta_acquire %llu leader dead_ext gen %llu clobbered prev_generation %llu next %llu",
+				  (unsigned long long)host_id,
+				  (unsigned long long)leader.owner_generation,
+				  (unsigned long long)prev_generation,
+				  (unsigned long long)next_generation);
+		} else {
+			/*
+			 * normal case when we restart after DEAD_EXT
+			 * was set in our lease.
+			 */
+			next_generation = leader.owner_generation + 1;
+
+			log_space(sp, "delta_acquire %llu leader dead_ext gen %llu normal prev_generation %llu next %llu",
+				  (unsigned long long)host_id,
+				  (unsigned long long)leader.owner_generation,
+				  (unsigned long long)prev_generation,
+				  (unsigned long long)next_generation);
+		}
+	} else {
+		/* common case */
+		next_generation = leader.owner_generation + 1;
+	}
+
 	new_ts = monotime();
-	leader.version = DELTA_DISK_VERSION_MAJOR | DELTA_DISK_VERSION_MINOR;
+	leader.version = version;
 	leader.timestamp = new_ts;
 	leader.io_timeout = (sp->io_timeout & 0x00FF);
 	leader.owner_id = host_id;
-	leader.owner_generation++;
+	leader.owner_generation = next_generation;
 	memcpy(leader.resource_name, our_host_name, NAME_ID_SIZE);
 	leader.checksum = 0; /* set below */
 
@@ -523,6 +681,8 @@ int delta_lease_acquire(struct task *task,
 		return rv;
 	}
 
+	delta_local_copy_write(sp, next_generation);
+
 	memcpy(&leader1, &leader, sizeof(struct leader_record));
 
 	if (no_delay) {
@@ -765,6 +925,37 @@ int delta_lease_renew(struct task *task,
 		return rv;
 	}
 
+	/*
+	 * Someone has set DEAD_EXT in our lease, likely along with
+	 * our previous generation.  This would be a race between us
+	 * joining (acquiring our lease), and another host declaring
+	 * us dead (ext) for our prior failure.  In this case, we want
+	 * to renew/write the values that we would have written if the
+	 * other host had not clobbered our previous write.
+	 *
+	 * What if they have set DEAD_EXT in our lease, with our
+	 * current generation number that we're renewing here?
+	 * This should not be allowed, and shouldn't happen.
+	 * If it does somehow, act like we have lost our lease by
+	 * returning the erro SANLK_RENEW_DEAD_EXT which is handled
+	 * like our lease is corrupted.
+	 */
+	if (leader.flags & LFL_DEAD_EXT) {
+		if (leader.owner_generation < leader_last->owner_generation) {
+			log_erros(sp, "delta_renew %llu late dead_ext gen %llu from %llu",
+				  (unsigned long long)leader.owner_id,
+				  (unsigned long long)leader.owner_generation,
+				  (unsigned long long)leader.unused1);
+			goto write_renew;
+		} else {
+			log_erros(sp, "delta_renew %llu current gen %llu is dead_ext from %llu",
+				  (unsigned long long)leader.owner_id,
+				  (unsigned long long)leader.owner_generation,
+				  (unsigned long long)leader.unused1);
+			return SANLK_RENEW_DEAD_EXT;
+		}
+	}
+
 	/* We can't always memcmp(&leader, leader_last) because previous writes
 	   may have timed out and we don't know if they were actually written
 	   or not.  We can definitely verify that we're still the owner,
@@ -793,6 +984,7 @@ int delta_lease_renew(struct task *task,
 		leader.io_timeout = (sp->io_timeout & 0x00FF);
 	}
 
+write_renew:
 	new_ts = monotime();
 
 	if (log_renewal_level != -1)
@@ -802,6 +994,9 @@ int delta_lease_renew(struct task *task,
 		log_erros(sp, "delta_renew timestamp too small");
 
 	leader.timestamp = new_ts;
+
+	leader.flags &= ~LFL_DEAD_EXT;
+
 	leader.checksum = 0; /* set below */
 
 	/* TODO: rename the leader fields */
@@ -927,11 +1122,15 @@ int delta_lease_init_host(struct task *task,
 	int align_size = 0;
 	int max_hosts = 0;
 	int write_io_timeout;
+	int no_timeout = (ls->flags & SANLK_LSF_NO_TIMEOUT) ? 1 : 0;
 	uint32_t checksum;
 	uint64_t new_ts;
 	uint64_t offset;
 	int rv;
 
+	if (no_timeout && com.create_delta_34)
+		return -EINVAL;
+
 	if (!io_timeout)
 		io_timeout = com.io_timeout;
 
@@ -965,8 +1164,10 @@ int delta_lease_init_host(struct task *task,
 
 	memset(&leader, 0, sizeof(struct leader_record));
 	leader.magic = DELTA_DISK_MAGIC;
-	leader.version = DELTA_DISK_VERSION_MAJOR | DELTA_DISK_VERSION_MINOR;
+	leader.version =  com.create_delta_34 ? DELTA_DISK_VERSION_34 : DELTA_DISK_VERSION_MAJOR | DELTA_DISK_VERSION_MINOR;
 	leader.flags = leader_align_flag_from_size(align_size);
+	if (no_timeout)
+		leader.flags |= LFL_NO_TIMEOUT;
 	leader.sector_size = sector_size;
 	leader.owner_id = ls->host_id;
 	leader.owner_generation = generation;
@@ -1029,9 +1230,13 @@ int delta_lease_init(struct task *task,
 	int align_size = 0;
 	int max_hosts = 0;
 	int write_io_timeout;
+	int no_timeout = (ls->flags & SANLK_LSF_NO_TIMEOUT) ? 1 : 0;
 	int i, rv;
 	uint32_t checksum;
 
+	if (no_timeout && com.create_delta_34)
+		return -EINVAL;
+
 	if (!io_timeout)
 		io_timeout = com.io_timeout;
 
@@ -1061,8 +1266,10 @@ int delta_lease_init(struct task *task,
 	for (i = 0; i < max_hosts; i++) {
 		memset(&leader, 0, sizeof(struct leader_record));
 		leader.magic = DELTA_DISK_MAGIC;
-		leader.version = DELTA_DISK_VERSION_MAJOR | DELTA_DISK_VERSION_MINOR;
+		leader.version =  com.create_delta_34 ? DELTA_DISK_VERSION_34 : DELTA_DISK_VERSION_MAJOR | DELTA_DISK_VERSION_MINOR;
 		leader.flags = leader_align_flag_from_size(align_size);
+		if (no_timeout)
+			leader.flags |= LFL_NO_TIMEOUT;
 		leader.sector_size = sector_size;
 		leader.max_hosts = 1;
 		leader.timestamp = LEASE_FREE;
@@ -1128,3 +1335,99 @@ int delta_lease_init(struct task *task,
 	return rv;
 }
 
+/* 
+ * Rewrites all delta leases in the lockspace like delta_lease_init(),
+ * but keeping existing leader values while modifying leader flags.
+ */
+
+int delta_lease_update_lockspace(struct task *task,
+		     struct sanlk_lockspace *ls,
+		     int io_timeout,
+		     struct sync_disk *disk)
+{
+	struct leader_record leader_end;
+	struct leader_record leader;
+	char *iobuf, **p_iobuf;
+	int iobuf_len;
+	int sector_size = 0;
+	int align_size = 0;
+	int max_hosts = 0;
+	int write_io_timeout;
+	int no_timeout = (ls->flags & SANLK_LSF_NO_TIMEOUT) ? 1 : 0;
+	int i, rv;
+	uint32_t checksum;
+
+	if (no_timeout && com.create_delta_34)
+		return -EINVAL;
+
+	if (!io_timeout)
+		io_timeout = com.io_timeout;
+
+	rv = sizes_from_flags(ls->flags, &sector_size, &align_size, &max_hosts, "LSF");
+	if (rv)
+		return rv;
+
+	if (!sector_size || !align_size || !max_hosts) {
+		log_error("delta_lease_update_lockspace requires LSF flags for size and alignment.");
+		return -EINVAL;
+	}
+
+	iobuf_len = align_size;
+
+	p_iobuf = &iobuf;
+
+	rv = posix_memalign((void *)p_iobuf, getpagesize(), iobuf_len);
+	if (rv)
+		return rv;
+
+	memset(iobuf, 0, iobuf_len);
+
+	rv = read_iobuf(disk->fd, disk->offset, iobuf, iobuf_len, task, io_timeout, NULL);
+	if (rv < 0)
+		goto out;
+
+	/* host_id N is block offset N-1 */
+
+	for (i = 0; i < max_hosts; i++) {
+		memcpy(&leader_end, iobuf + (i * sector_size), sizeof(struct leader_record));
+
+		leader_record_in(&leader_end, &leader);
+
+		if (leader.magic != DELTA_DISK_MAGIC)
+			continue;
+
+		if (no_timeout)
+			leader.flags |= LFL_NO_TIMEOUT;
+		else
+			leader.flags &= ~LFL_NO_TIMEOUT;
+
+		leader.checksum = 0; /* set below */
+
+		leader_record_out(&leader, &leader_end);
+
+		/*
+		 * N.B. must compute checksum after the data has been byte swapped.
+		 */
+		checksum = leader_checksum(&leader_end);
+		leader.checksum = checksum;
+		leader_end.checksum = cpu_to_le32(checksum);
+
+		memcpy(iobuf + (i * sector_size), &leader_end, sizeof(struct leader_record));
+	}
+	
+	if (com.write_init_io_timeout)
+		write_io_timeout = com.write_init_io_timeout;
+	else
+		write_io_timeout = io_timeout;
+
+	rv = write_iobuf(disk->fd, disk->offset, iobuf, iobuf_len, task, write_io_timeout, NULL);
+	if (rv < 0)
+		goto out;
+
+ out:
+	if (rv != SANLK_AIO_TIMEOUT)
+		free(iobuf);
+
+	return rv;
+}
+
