@@ -132,6 +132,7 @@ static int do_paxos_action(int action, struct task *task, int io_timeout, struct
 	int max_hosts = 0;
 	int disks_len, token_len;
 	int j, rv = 0;
+	uint64_t max_mbal;
 
 	if (!io_timeout)
 		io_timeout = com.io_timeout;
@@ -224,6 +225,24 @@ static int do_paxos_action(int action, struct task *task, int io_timeout, struct
 		rv = paxos_lease_release(task, token, NULL, &leader, leader_ret);
 		break;
 
+	case ACT_READ:
+		if (!sector_size || !align_size) {
+			rv = direct_read_leader_sizes(task, &token->disks[0], &sector_size, &align_size);
+			if (rv < 0)
+				break;
+		}
+
+		token->sector_size = sector_size;
+		token->align_size = align_size;
+		token->host_id = 1;
+
+		/* 
+		read all the initial values needed to start disk paxos, including leader record and all dblocks.
+		TODO: reading the owners of resource will be implemented in the future.
+		*/
+		rv = paxos_lease_read(task, token, 0, &leader, &max_mbal, "direct_read", 1);
+		break;
+
 	case ACT_READ_LEADER:
 		if (!sector_size)
 			sector_size = 4096;
@@ -314,6 +333,8 @@ static int do_delta_action(int action,
 
 	if (!io_timeout)
 		io_timeout = com.io_timeout;
+	if (!io_timeout)
+		io_timeout = DEFAULT_IO_TIMEOUT;
 
 	rv = sizes_from_flags(ls->flags, &sector_size, &align_size, &max_hosts, "LSF");
 	if (rv)
@@ -427,6 +448,7 @@ static int do_delta_action(int action,
 					 &leader);
 		break;
 	case ACT_READ_LEADER:
+	case ACT_READ:
 		if (!sector_size || !align_size) {
 			rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
 			if (rv < 0)
@@ -528,6 +550,25 @@ int direct_write_resource(struct task *task, struct sanlk_resource *res,
 			       NULL, NULL);
 }
 
+int direct_read(struct task *task,
+		       int io_timeout,
+		       struct sanlk_lockspace *ls,
+		       struct sanlk_resource *res,
+		       struct leader_record *leader_ret)
+{
+	int rv = -1;
+
+	if (ls && ls->host_id_disk.path[0])
+		rv = do_delta_action(ACT_READ, task, io_timeout, ls, 0, 0, NULL, NULL, leader_ret);
+
+	else if (res)
+		rv = do_paxos_action(ACT_READ, task, io_timeout, res,
+				     0, 0,
+				     0, 0,
+				     NULL, leader_ret);
+	return rv;
+}
+
 int direct_read_leader(struct task *task,
 		       int io_timeout,
 		       struct sanlk_lockspace *ls,
@@ -751,12 +792,13 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 				memcpy(sname, lr->space_name, NAME_ID_SIZE);
 				memcpy(rname, lr->resource_name, NAME_ID_SIZE);
 
-				printf("%08llu %36s %48s %010llu %04llu %04llu",
+				printf("%08llu %36s %48s %010llu %04llu %04llu 0x%08llx",
 					(unsigned long long)(start_offset + ((sector_nr + i) * sector_size)),
 					sname, rname,
 					(unsigned long long)lr->timestamp,
 					(unsigned long long)lr->owner_id,
-					(unsigned long long)lr->owner_generation);
+					(unsigned long long)lr->owner_generation,
+					(unsigned long long)lr->flags);
 
 				if (force_mode) {
 					bitmap = (char *)lr_end + LEADER_RECORD_MAX;
