@@ -98,7 +98,7 @@ static void setup_task_lib(struct task *task, int use_aio)
 
 
 int sanlock_direct_write_lockspace(struct sanlk_lockspace *ls, int max_hosts_unused,
-                                   uint32_t flags GNUC_UNUSED, uint32_t io_timeout)
+                                   uint32_t flags, uint32_t io_timeout)
 {
 	struct task task;
 	int rv;
