@@ -19,8 +19,11 @@
  * paxos minor version changes:
  * 4: add ALIGN leader flags
  *
- * delta minor version changes:
- * 4: add ALIGN leader flags
+ * delta version changes:
+ * 3.4: add ALIGN leader flags
+ * 4.0: add NO_TIMEOUT and DEAD_EXT leader flags
+ *      breaks compat with older versions, because they would take
+ *      a NO_TIMEOUT lease after dead timeout when they shouldn't.
  */
 
 #define PAXOS_DISK_MAGIC 0x06152010
@@ -29,8 +32,11 @@
 #define PAXOS_DISK_VERSION_MINOR 0x00000004
 
 #define DELTA_DISK_MAGIC 0x12212010
-#define DELTA_DISK_VERSION_MAJOR 0x00030000
-#define DELTA_DISK_VERSION_MINOR 0x00000004
+#define DELTA_DISK_VERSION_MAJOR 0x00040000
+#define DELTA_DISK_VERSION_MINOR 0x00000000
+
+/* 3.4 does not support NO_TIMEOUT leader flag. */
+#define DELTA_DISK_VERSION_34 0x00030004
 
 /* for all disk structures:
    uint64 aligned on 8 byte boundaries,
@@ -53,6 +59,8 @@
 
 /* leader_record flags */
 #define LFL_SHORT_HOLD 0x00000001
+#define LFL_NO_TIMEOUT 0x00000002
+#define LFL_DEAD_EXT   0x00000004
 /* skip ahead in flag numbers so these align flags match other defines */
 #define LFL_ALIGN_1M   0x00000010
 #define LFL_ALIGN_2M   0x00000020
