@@ -461,6 +461,31 @@ void check_other_leases(struct space *sp, char *buf)
 			/* log warning for each generation of a host_id */
 			hs->warned_fail = 0;
 			hs->warned_dead = 0;
+			hs->warned_dead_ext = 0;
+		}
+
+		if (!hs->lease_bad) {
+			hs->no_timeout = (leader->flags & LFL_NO_TIMEOUT) ? 1 : 0;
+			hs->dead_ext = (leader->flags & LFL_DEAD_EXT) ? 1 : 0;
+
+			if ((hs->no_timeout || hs->dead_ext) && (leader->version <= DELTA_DISK_VERSION_34)) {
+				hs->no_timeout = 0;
+				hs->dead_ext = 0;
+				if (!hs->warned_dead_ext)
+					log_erros(sp, "check_other_lease %llu %llu version 0x%x should not have flags 0x%x",
+						  (unsigned long long)leader->owner_id,
+						  (unsigned long long)leader->owner_generation,
+						  leader->version, leader->flags);
+				hs->warned_dead_ext = 1;
+			}
+
+			if (hs->dead_ext && !hs->warned_dead_ext) {
+				log_warns(sp, "host %llu %llu %llu is dead (ext)",
+					  (unsigned long long)leader->owner_id,
+					  (unsigned long long)leader->owner_generation,
+					  (unsigned long long)leader->timestamp);
+				hs->warned_dead_ext = 1;
+			}
 		}
 
 		/*
@@ -518,7 +543,7 @@ void check_other_leases(struct space *sp, char *buf)
 				hs->warned_fail = 1;
 			}
 
-			if (!hs->warned_dead &&
+			if (!hs->warned_dead && !hs->no_timeout &&
 			    (hs->last_check - last > other_host_dead_seconds)) {
 				log_warns(sp, "host %llu %llu %llu is dead",
 					 (unsigned long long)hs->owner_id,
@@ -565,6 +590,9 @@ void check_other_leases(struct space *sp, char *buf)
 			hs->owner_generation = leader->owner_generation;
 			memcpy(hs->owner_name, leader->resource_name, NAME_ID_SIZE);
 			hs->io_timeout = leader->io_timeout;
+			hs->warned_fail = 0;
+			hs->warned_dead = 0;
+			hs->warned_dead_ext = 0;
 		}
 
 		hs->timestamp = leader->timestamp;
@@ -661,7 +689,9 @@ int check_our_lease(struct space *sp, int *check_all, char *check_buf)
 	id_renewal_warn_seconds = calc_id_renewal_warn_seconds(sp->io_timeout);
 
 	if (gap >= id_renewal_fail_seconds) {
-		log_erros(sp, "check_our_lease failed %d", gap);
+		log_erros(sp, "check_our_lease failed %d%s", gap, sp->no_timeout ? " (no_timeout)" : "");
+		if (sp->no_timeout)
+			return 0;
 		return -1;
 	}
 
@@ -693,6 +723,7 @@ static int corrupt_result(int result)
 	case SANLK_LEADER_SECTORSIZE:
 	case SANLK_LEADER_LOCKSPACE:
 	case SANLK_LEADER_CHECKSUM:
+	case SANLK_RENEW_DEAD_EXT:
 		return result;
 	default:
 		return 0;
@@ -897,16 +928,18 @@ static void *lockspace_thread(void *arg_in)
 	}
 	opened = 1;
 
-	rv = delta_read_lockspace_sizes(&task, &sp->host_id_disk, sp->io_timeout, &sector_size, &align_size);
+	rv = delta_read_lockspace_sizes(&task, &sp->host_id_disk, sp->host_id, sp->io_timeout, &sector_size, &align_size);
 	if (rv < 0) {
-		log_erros(sp, "failed to read device to find sector size error %d %s", rv, sp->host_id_disk.path);
+		log_erros(sp, "failed to read device to find sector size for host_id %llu error %d %s",
+			  (unsigned long long)sp->host_id, rv, sp->host_id_disk.path);
 		acquire_result = rv;
 		delta_result = -1;
 		goto set_status;
 	}
 
 	if ((sector_size != 512) && (sector_size != 4096)) {
-		log_erros(sp, "failed to get valid sector size %d %s", sector_size, sp->host_id_disk.path);
+		log_erros(sp, "failed to get valid sector size %d for host_id %llu %s",
+			  sector_size, (unsigned long long)sp->host_id, sp->host_id_disk.path);
 		acquire_result = SANLK_LEADER_SECTORSIZE;
 		delta_result = -1;
 		goto set_status;
@@ -1012,13 +1045,17 @@ static void *lockspace_thread(void *arg_in)
 		sp->lease_status.renewal_last_success = last_success;
 	/* First renewal entry shows the acquire time with 0 latencies. */
 	save_renewal_history(sp, delta_result, last_success, 0, 0);
+
+	if (acquire_result >= 0) {
+		sp->host_generation = leader.owner_generation;
+		sp->version = leader.version;
+		sp->no_timeout = (leader.flags & LFL_NO_TIMEOUT) ? 1 : 0;
+	}
 	pthread_mutex_unlock(&sp->mutex);
 
 	if (acquire_result < 0)
 		goto out;
 
-	sp->host_generation = leader.owner_generation;
-
 	while (1) {
 		pthread_mutex_lock(&sp->mutex);
 		stop = sp->thread_stop;
@@ -1536,6 +1573,9 @@ int get_lockspaces(char *buf, int *len, int *count, int maxlen)
 			else if (i == 2)
 				ls->flags |= SANLK_LSF_ADD;
 
+			if (sp->no_timeout)
+				ls->flags |= SANLK_LSF_NO_TIMEOUT;
+
 			*len += sizeof(struct sanlk_lockspace);
 
 			ls++;
@@ -1611,14 +1651,32 @@ int get_lockspaces(char *buf, int *len, int *count, int maxlen)
  * After 80 seconds, we'd return FAIL.  After 140 seconds we'd return DEAD.
  */
 
-uint32_t get_host_flag(struct space *sp, struct host_status *hs)
+uint32_t get_host_flag(uint64_t local_host_id, struct host_status *hs)
 {
-	uint32_t other_io_timeout;
-	int other_host_fail_seconds, other_host_dead_seconds;
+	uint32_t other_io_timeout = hs->io_timeout;
+	int other_host_fail_seconds = calc_id_renewal_fail_seconds(other_io_timeout);
+	int other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
 
-	/* we know we are alive */
-	if (sp->host_id == hs->owner_id)
-		return SANLK_HOST_LIVE;
+	if (hs->dead_ext ||
+	    (hs->found_dead_ext_gen == hs->owner_generation) ||
+	    (hs->wrote_dead_ext_gen == hs->owner_generation)) {
+		uint32_t flag = SANLK_HOST_DEAD | SANLK_HOST_DEAD_EXT;
+
+		if (hs->no_timeout)
+			return flag;
+
+		/* a host can be DEAD from both DEAD_EXT flag and host_dead_sec timeout,
+		   copy the timeout-based DEAD checks from below. */
+
+		if (!hs->last_live) {
+			if (hs->last_check - hs->first_check > other_host_dead_seconds)
+				flag |= SANLK_HOST_DEAD_SEC;
+		} else {
+			if (hs->last_check - hs->last_live > other_host_dead_seconds)
+				flag |= SANLK_HOST_DEAD_SEC;
+		}
+		return flag;
+	}
 
 	/* See sh_holder_live() for same/similar checks */
 
@@ -1628,6 +1686,10 @@ uint32_t get_host_flag(struct space *sp, struct host_status *hs)
 	if (!hs->last_check)
 		return SANLK_HOST_UNKNOWN;
 
+	/* The most recent data is older than the fail time. */
+	if (monotime() - hs->last_check > other_host_fail_seconds)
+		return SANLK_HOST_UNKNOWN | SANLK_HOST_UNKNOWN_OLD;
+
 	if (!hs->timestamp)
 		return SANLK_HOST_FREE;
 
@@ -1637,21 +1699,17 @@ uint32_t get_host_flag(struct space *sp, struct host_status *hs)
 	if (hs->last_check == hs->last_live)
 		return SANLK_HOST_LIVE;
 
-	other_io_timeout = hs->io_timeout;
-	other_host_fail_seconds = calc_id_renewal_fail_seconds(other_io_timeout);
-	other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
-
 	if (!hs->last_live) {
 		if (hs->last_check - hs->first_check <= other_host_fail_seconds)
 			return SANLK_HOST_UNKNOWN;
-		if (hs->last_check - hs->first_check > other_host_dead_seconds)
+		if (!hs->no_timeout && (hs->last_check - hs->first_check > other_host_dead_seconds))
 			return SANLK_HOST_DEAD;
 		if (hs->last_check - hs->first_check > other_host_fail_seconds)
 			return SANLK_HOST_FAIL;
 		/* shouldn't be possible */
 		return SANLK_HOST_UNKNOWN;
 	} else {
-		if (hs->last_check - hs->last_live > other_host_dead_seconds)
+		if (!hs->no_timeout && (hs->last_check - hs->last_live > other_host_dead_seconds))
 			return SANLK_HOST_DEAD;
 		if (hs->last_check - hs->last_live > other_host_fail_seconds)
 			return SANLK_HOST_FAIL;
@@ -1680,7 +1738,7 @@ int get_host_local(struct sanlk_lockspace *ls, struct sanlk_host *host)
 	host->generation = sp->host_generation;
 	host->timestamp = hs->timestamp;
 	host->io_timeout = hs->io_timeout;
-	host->flags = get_host_flag(sp, hs);
+	host->flags = get_host_flag(sp->host_id, hs);
  out:
 	pthread_mutex_unlock(&spaces_mutex);
 
@@ -1738,7 +1796,7 @@ int get_hosts(struct sanlk_lockspace *ls, char *buf, int *len, int *count, int m
 		host->generation = hs->owner_generation;
 		host->timestamp = hs->timestamp;
 		host->io_timeout = hs->io_timeout;
-		host->flags = get_host_flag(sp, hs);
+		host->flags = get_host_flag(sp->host_id, hs);
 
 		*len += sizeof(struct sanlk_host);
 
@@ -1752,6 +1810,178 @@ int get_hosts(struct sanlk_lockspace *ls, char *buf, int *len, int *count, int m
 	return rv;
 }
 
+/*
+  A starts PR key A.1 (no local prev_gen)
+  A writes delta lease A.1 (sets local prev_gen 1)
+  A lock R1 writes paxos lease owner A.1
+  A.1 fails (enters host state FAIL)
+  B lock R1 reads paxos lease owner A.1 (lock fails, owner A.1)
+  B sees A.1 is in FAIL state
+  B removes PR key A.1
+  A is restarting
+  A starts PR key A.2 (sees local prev_gen 1)
+  B set_host reads delta lease A.1
+  A start writes delta lease A.2 (sets local prev_gen 2)
+  A lock R2 writes paxos lease owner A.2
+  A.2 fails (enters host state FAIL)
+  B set_host writes delta lease A.1.DEAD_EXT (delta lease A.2 lost)
+  B lock R1 retry succeeds because paxos is A.1 and delta is A.1.DEAD_EXT
+  B lock R2 reads paxos lease with owner A.2 (lock fails, owner A.2)
+  B sees A.2 is in FAIL state, or sees delta A.1.DEAD_EXT (does this matter?)
+  B removes PR key A.2
+  A is restarting
+  A starts PR key A.3 (sees local prev_gen 2)
+  A start reads delta lease A.1.DEAD_EXT
+  B set_host reads delta lease A.1.DEAD_EXT
+  A start writes delta lease A.3 (sees local prev_gen 2)
+    (without local prev_gen, it would write delta lease A.2)
+  A lock R3 writes paxos lease owner A.3
+  B set_host writes delta lease A.2.DEAD_EXT
+  B lock R2 retry succeeds because paxos is A.2 and delta is A.2.DEAD_EXT
+    (if B lock R2 retry was after A renew A.3, it would succeed because
+    paxos is A.2 and delta is A.3)
+  A renew writes delta lease A.3
+*/
+
+int lockspace_set_host_dead_ext(struct task *task, struct sanlk_lockspace *ls, struct sanlk_host *hs,
+				uint32_t io_timeout)
+{
+	struct space *sp;
+	struct sync_disk sd;
+	struct leader_record leader;
+	int max_hosts;
+	int sector_size;
+	uint64_t local_host_id;
+	uint64_t local_generation;
+	int found_dead_ext = 0;
+	int wrote_dead_ext = 0;
+	int rv = -1;
+
+	if (!ls->host_id)
+		return -EINVAL;
+
+	pthread_mutex_lock(&spaces_mutex);
+	sp = _search_space(ls->name, NULL, 0, &spaces, NULL, NULL, NULL);
+	if (!sp) {
+		pthread_mutex_unlock(&spaces_mutex);
+		return -ENOENT;
+	}
+	max_hosts = sp->max_hosts;
+	sector_size = sp->sector_size;
+	local_host_id = sp->host_id;
+	local_generation = sp->host_generation;
+	if (!io_timeout)
+		io_timeout = sp->io_timeout;
+	memcpy(&sd, &sp->host_id_disk, sizeof(struct sync_disk));
+	pthread_mutex_unlock(&spaces_mutex);
+
+	log_debug("lockspace_set_host_dead_ext %.48s host %llu %llu",
+		  ls->name, (unsigned long long)hs->host_id,
+		  (unsigned long long)hs->generation);
+
+	if (ls->host_id_disk.path[0] && strcmp(sd.path, ls->host_id_disk.path))
+		return -EINVAL;
+
+	if (ls->host_id && (ls->host_id != local_host_id))
+		return -EINVAL;
+
+	if (hs->host_id > max_hosts)
+		return -EINVAL;
+
+	rv = open_disk(&sd);
+	if (rv < 0) {
+		log_error("lockspace_set_host_dead_ext %.48s host %llu %llu open_disk %d",
+			  ls->name, (unsigned long long)hs->host_id,
+			  (unsigned long long)hs->generation, rv);
+		return rv;
+	}
+
+	rv = delta_lease_leader_read(task, sector_size, io_timeout, &sd,
+				     ls->name, hs->host_id, &leader, "set_host_dead_ext");
+	if (rv < 0) {
+		log_error("lockspace_set_host_dead_ext %.48s host %llu %llu read %d",
+			  ls->name, (unsigned long long)hs->host_id,
+			  (unsigned long long)hs->generation, rv);
+		goto out;
+	}
+
+	if (leader.version <= DELTA_DISK_VERSION_34) {
+		log_error("lockspace_set_host_dead_ext %.48s host %llu %llu version incompat 0x%x",
+			  ls->name, (unsigned long long)hs->host_id,
+			  (unsigned long long)hs->generation, leader.version);
+		rv = SANLK_VERSION_INCOMPAT;
+		goto out;
+	}
+
+	if (hs->generation && (leader.owner_generation > hs->generation)) {
+		log_error("lockspace_set_host_dead_ext %.48s host %llu %llu leader gen %llu",
+			  ls->name, (unsigned long long)hs->host_id,
+			  (unsigned long long)hs->generation,
+			  (unsigned long long)leader.owner_generation);
+		rv = SANLK_WRONG_GENERATION;
+		goto out;
+	}
+
+	if (leader.flags & LFL_DEAD_EXT) {
+		found_dead_ext = 1;
+		goto save;
+	}
+
+	leader.flags |= LFL_DEAD_EXT;
+
+	if (hs->generation)
+		leader.owner_generation = hs->generation;
+
+	/* for debugging only */
+	leader.unused1 = (local_host_id << 16) | (local_generation & 0x0000FFFF);
+
+	rv = delta_lease_leader_clobber(task, io_timeout, &sd, hs->host_id, &leader, "set_host_dead_ext");
+	if (rv < 0) {
+		log_error("lockspace_set_host_dead_ext %.48s host %llu %llu write %d",
+			  ls->name, (unsigned long long)hs->host_id,
+			  (unsigned long long)hs->generation, rv);
+		goto out;
+	}
+
+	wrote_dead_ext = 1;
+
+	/*
+	 * Note: multiple hosts may all clobber this lease to set DEAD_EXT
+	 * at the same time, with the last one winning.  So multiple hosts
+	 * may show wrote_dead_ext_gen.
+	 *
+	 * If the target host is restarting at the same time, this may
+	 * clobber the delta acquire with the next gen from the owner.
+	 * In that case the owner's next renewal will see its own delta
+	 * lease go backward to the prev gen with DEAD_EXT set.  The
+	 * delta renewal can ignore that and go ahead with its next
+	 * renewal update.
+	 */
+
+ save:
+	pthread_mutex_lock(&spaces_mutex);
+	sp = _search_space(ls->name, NULL, 0, &spaces, NULL, NULL, NULL);
+	if (!sp) {
+		pthread_mutex_unlock(&spaces_mutex);
+		log_error("lockspace_set_host_dead_ext %.48s host %llu %llu sp removed",
+			  ls->name, (unsigned long long)hs->host_id,
+			  (unsigned long long)hs->generation);
+		rv = -ENOENT;
+		goto out;
+	}
+	if (found_dead_ext)
+		sp->host_status[hs->host_id-1].found_dead_ext_gen = leader.owner_generation;
+	if (wrote_dead_ext)
+		sp->host_status[hs->host_id-1].wrote_dead_ext_gen = leader.owner_generation;
+	pthread_mutex_unlock(&spaces_mutex);
+
+	rv = 0;
+ out:
+	close_disks(&sd, 1);
+	return rv;
+
+}
+
 int lockspace_set_config(struct sanlk_lockspace *ls, GNUC_UNUSED uint32_t flags, uint32_t cmd, uint32_t data)
 {
 	struct space *sp;
