@@ -1714,7 +1714,7 @@ static int _lease_read_num(struct task *task,
  * values from it.
  */
 
-static int paxos_lease_read(struct task *task, struct token *token, uint32_t flags,
+int paxos_lease_read(struct task *task, struct token *token, uint32_t flags,
 			    struct leader_record *leader_ret,
 			    uint64_t *max_mbal, const char *caller, int log_bk_vals)
 {
@@ -1836,6 +1836,7 @@ int paxos_lease_acquire(struct task *task,
 	uint64_t num_mbal;
 	uint64_t our_mbal;
 	uint64_t other_max_mbal;
+	uint64_t set_dead_ext_gen = 0;
 	int copy_cur_leader;
 	int disk_open = 0;
 	int error, rv, us;
@@ -1972,15 +1973,93 @@ int paxos_lease_acquire(struct task *task,
 	}
 
 	log_token(token, "paxos_acquire owner %llu %llu %llu "
-		  "host_status %llu %llu %llu wait_start %llu",
+		  "host_status %llu %llu %llu dead_ext %d wait_start %llu",
 		  (unsigned long long)cur_leader.owner_id,
 		  (unsigned long long)cur_leader.owner_generation,
 		  (unsigned long long)cur_leader.timestamp,
 		  (unsigned long long)hs.owner_id,
 		  (unsigned long long)hs.owner_generation,
 		  (unsigned long long)hs.timestamp,
+		  hs.dead_ext,
 		  (unsigned long long)wait_start);
 
+	/*
+	 * If an external entity has said that the owner is dead,
+	 * then we don't need to use the delta lease timeout of
+	 * the owner as the basis for deciding the owner is dead.
+	 *
+	 * Note: hs from host_info() was last updated from our
+	 * previous renewal, and may not reflect the latest
+	 * info from the owner's delta lease.
+	 */
+	if (hs.dead_ext && (hs.owner_generation == cur_leader.owner_generation)) {
+		log_token(token, "paxos_acquire owner %llu %llu %llu "
+			  "delta %llu %llu %llu host_status is dead_ext",
+			  (unsigned long long)cur_leader.owner_id,
+			  (unsigned long long)cur_leader.owner_generation,
+			  (unsigned long long)cur_leader.timestamp,
+			  (unsigned long long)host_id_leader.owner_id,
+			  (unsigned long long)host_id_leader.owner_generation,
+			  (unsigned long long)host_id_leader.timestamp);
+		goto run;
+	}
+
+	/*
+	 * If the paxos leader shows gen N and host_id leader shows
+	 * gen N-1 with DEAD_EXT, then the host's latest delta lease was
+	 * clobbered by another host setting DEAD_EXT with the prev gen,
+	 * causing the generation to go back one.  Return an error specific
+	 * to this case and retry.  Eventually we should see the owner delta
+	 * lease gen N with DEAD_EXT or gen N+1.
+	 */
+	if (hs.dead_ext && (hs.owner_generation < cur_leader.owner_generation))  {
+		log_errot(token, "paxos_acquire owner %llu %llu %llu reverted delta %llu %llu %llu dead_ext",
+			  (unsigned long long)cur_leader.owner_id,
+			  (unsigned long long)cur_leader.owner_generation,
+			  (unsigned long long)cur_leader.timestamp,
+			  (unsigned long long)host_id_leader.owner_id,
+			  (unsigned long long)host_id_leader.owner_generation,
+			  (unsigned long long)host_id_leader.timestamp);
+		memcpy(leader_ret, &cur_leader, sizeof(struct leader_record));
+		return SANLK_ACQUIRE_OWNER_GEN_REVERTED;
+	}
+
+	/*
+	 * These would have been set by a local entity calling
+	 * set_host with dead_ext on the failed host.  If a local
+	 * entity has not does this, then eventually we would see
+	 * the hs.dead_ext set from another host.
+	 */
+	if (hs.found_dead_ext_gen)
+		set_dead_ext_gen = hs.found_dead_ext_gen;
+	else if (hs.wrote_dead_ext_gen)
+		set_dead_ext_gen = hs.wrote_dead_ext_gen;
+
+	/*
+	 * A local entity called set_host+dead_ext on this host,
+	 * so we know it's dead from that, even if our local
+	 * renewal thread has not yet seen it and set hs.dead_ext
+	 * in check_other_leases.
+	 */
+	if (set_dead_ext_gen && (set_dead_ext_gen == cur_leader.owner_generation)) {
+		log_token(token, "paxos_acquire owner %llu %llu %llu "
+			  "delta %llu %llu %llu is dead_ext (found %llu wrote %llu)",
+			  (unsigned long long)cur_leader.owner_id,
+			  (unsigned long long)cur_leader.owner_generation,
+			  (unsigned long long)cur_leader.timestamp,
+			  (unsigned long long)host_id_leader.owner_id,
+			  (unsigned long long)host_id_leader.owner_generation,
+			  (unsigned long long)host_id_leader.timestamp,
+			  (unsigned long long)hs.found_dead_ext_gen,
+			  (unsigned long long)hs.wrote_dead_ext_gen);
+		goto run;
+	}
+
+	/*
+	 * Loop waiting for the owner of this paxos lease to be dead by timeout.
+	 * The paxos lease owner will be dead when the owner's delta lease expires,
+	 * indicating that the owner's watchdog has fired.
+	 */
 	while (1) {
 		error = delta_lease_leader_read(task, ls_sector_size, token->io_timeout,
 						&host_id_disk,
@@ -2036,6 +2115,24 @@ int paxos_lease_acquire(struct task *task,
 			goto skip_live_check;
 		}
 
+		/*
+		 * The local renewal thread has not yet read the owner's delta
+		 * lease since DEAD_EXT was set, so hs.dead_ext was not seen above.
+		 * But we've read the owner's delta lease here, and see it here.
+		 */
+		if ((host_id_leader.flags & LFL_DEAD_EXT) &&
+		    (host_id_leader.owner_generation == cur_leader.owner_generation)) {
+			log_token(token, "paxos_acquire owner %llu %llu %llu "
+				  "delta %llu %llu %llu read flag dead_ext",
+				  (unsigned long long)cur_leader.owner_id,
+				  (unsigned long long)cur_leader.owner_generation,
+				  (unsigned long long)cur_leader.timestamp,
+				  (unsigned long long)host_id_leader.owner_id,
+				  (unsigned long long)host_id_leader.owner_generation,
+				  (unsigned long long)host_id_leader.timestamp);
+			goto run;
+		}
+
 		/*
 		 * Check if the owner is alive:
 		 *
@@ -2114,6 +2211,28 @@ int paxos_lease_acquire(struct task *task,
 		other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
 
 		if (now - wait_start > other_host_dead_seconds) {
+
+			/*
+			 * This host would be considered dead if it weren't using NO_TIMEOUT,
+			 * and no dead_ext has been set for it yet allowing us to procede.
+			 */
+			if (host_id_leader.flags & LFL_NO_TIMEOUT) {
+				log_token(token, "paxos_acquire owner %llu %llu %llu "
+					  "delta %llu %llu %llu no_timeout_dead %llu-%llu>%d",
+					  (unsigned long long)cur_leader.owner_id,
+					  (unsigned long long)cur_leader.owner_generation,
+				          (unsigned long long)cur_leader.timestamp,
+				          (unsigned long long)host_id_leader.owner_id,
+				          (unsigned long long)host_id_leader.owner_generation,
+				          (unsigned long long)host_id_leader.timestamp,
+				          (unsigned long long)now,
+				          (unsigned long long)wait_start,
+				          other_host_dead_seconds);
+				memcpy(leader_ret, &cur_leader, sizeof(struct leader_record));
+				error = SANLK_ACQUIRE_OWNED_NO_TIMEOUT;
+				goto out;
+			}
+
 			log_token(token, "paxos_acquire owner %llu %llu %llu "
 				  "delta %llu %llu %llu dead %llu-%llu>%d",
 				  (unsigned long long)cur_leader.owner_id,
@@ -2133,6 +2252,7 @@ int paxos_lease_acquire(struct task *task,
 				  (unsigned long long)cur_leader.owner_id,
 				  (unsigned long long)cur_leader.owner_generation,
 				  (unsigned long long)cur_leader.timestamp);
+			memcpy(leader_ret, &cur_leader, sizeof(struct leader_record));
 			error = SANLK_ACQUIRE_OWNED_RETRY;
 			goto out;
 		}
@@ -2169,6 +2289,8 @@ int paxos_lease_acquire(struct task *task,
 			goto restart;
 		}
 	}
+	/* end of loop waiting for the owner to be dead */
+
  run:
 	/*
 	 * Use the disk paxos algorithm to attempt to commit a new leader.
