@@ -56,6 +56,10 @@ int paxos_read_buf(struct task *task,
                    struct token *token,
                    char **buf_out);
 
+int paxos_lease_read(struct task *task, struct token *token, uint32_t flags,
+			    struct leader_record *leader_ret,
+			    uint64_t *max_mbal, const char *caller, int log_bk_vals);
+
 int paxos_verify_leader(struct token *token,
                          struct sync_disk *disk,
                          struct leader_record *lr,
