@@ -395,6 +395,20 @@ static int sh_holder_live(char *lockspace_name, uint32_t space_id, uint64_t host
 		return 1;
 	}
 
+	if (hs.dead_ext ||
+	    (hs.found_dead_ext_gen == hs.owner_generation) ||
+	    (hs.wrote_dead_ext_gen == hs.owner_generation)) {
+		log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu DEAD_EXT - %d,%llu,%llu",
+			(unsigned long long)host_id,
+			(unsigned long long)hs.owner_generation,
+			(unsigned long long)hs.timestamp,
+			(unsigned long long)mb_gen,
+			hs.dead_ext,
+			(unsigned long long)hs.found_dead_ext_gen,
+			(unsigned long long)hs.wrote_dead_ext_gen);
+		return 0;
+	}
+
 	if (!hs.first_check) {
 		log_sid(space_id, "sh_holder %llu mb_gen %llu LIVE - host_info not yet checked",
 			(unsigned long long)host_id, (unsigned long long)mb_gen);
@@ -2014,7 +2028,11 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 	r->sector_size = token->sector_size;
 	r->align_size = token->align_size;
 
-	if (rv == SANLK_ACQUIRE_IDLIVE || rv == SANLK_ACQUIRE_OWNED || rv == SANLK_ACQUIRE_OTHER) {
+	if (rv == SANLK_ACQUIRE_IDLIVE ||
+	    rv == SANLK_ACQUIRE_OWNED ||
+	    rv == SANLK_ACQUIRE_OTHER ||
+	    rv == SANLK_ACQUIRE_OWNED_RETRY ||
+	    rv == SANLK_ACQUIRE_OWNED_NO_TIMEOUT) {
 		/*
 		 * Another host owns the lease.  They may be holding for
 		 * only a short time while getting a shared lease.
