@@ -21,9 +21,12 @@
 /* inq flags */
 #define SANLK_INQ_WAIT		0x00000001
 
-/* write flags */
+/* write_resource flags */
 #define SANLK_WRITE_CLEAR	0x00000001 /* subsequent read will return error */
 
+/* sanlock_write_lockspace */
+#define SANLK_WRITE_LS_FLAGS 0x00000001 /* update leader flags in lockspace delta leases */
+
 /* get_hosts flags */
 #define SANLK_GET_HOST_LOCAL	0x00000001
 
@@ -35,6 +38,10 @@
 #define SANLK_HOST_DEAD    0x00000005
 #define SANLK_HOST_MASK    0x0000000F /* select SANLK_HOST_ from flags */
 
+#define SANLK_HOST_DEAD_EXT    0x00000010 /* DEAD_EXT delta lease flag */
+#define SANLK_HOST_DEAD_SEC    0x00000020 /* host_dead_seconds timeout */
+#define SANLK_HOST_UNKNOWN_OLD 0x00000040 /* data is too old */
+
 /*
  * add_lockspace returns:
  * 0: the lockspace has been added successfully
@@ -129,6 +136,25 @@ int sanlock_get_hosts(const char *ls_name, uint64_t host_id,
 		      struct sanlk_host **hss, int *hss_count,
 		      uint32_t flags);
 
+/*
+ * SANLK_SET_HOST_DEAD_EXT
+ * -----------------------
+ * Set the DEAD_EXT flag in a host's delta_lease.
+ * *host_data is *sanlk_host, which specifies the host_id to change.
+ * The lockspace arg specifies the lockspace to operate on (if host_id
+ * is specified in the lockspace arg, it should match the local host_id.)
+ * The delta_lease leader record will be read, modified,
+ * and written back with the DEAD_EXT flag set.
+ * If a generation number is set in sanlk_host, that generation
+ * number will be set in the leader record that is written,
+ * regardless of the generation number found in the leader
+ * record when read.
+ */
+
+#define SANLK_SET_HOST_DEAD_EXT 0x00000001
+
+int sanlock_set_host(struct sanlk_lockspace *ls, uint32_t cmd, uint32_t flags, uint32_t io_timeout, void *host_data);
+
 /*
  * set_config cmd values
  *
@@ -237,11 +263,26 @@ int sanlock_init(struct sanlk_lockspace *ls,
  * the first host_id (1) (the first record at offset) is the last
  * leader record written, so read_lockspace of host_id 1 will fail
  * until the entire write_lockspace is complete.
+ *
+ * SANLK_WRITE_LS_FLAGS:
+ * Update the leader flags of each delta lease in the lockspace.
+ * LFL_NO_TIMEOUT is set if SANLK_LSF_NO_TIMEOUT is set in the
+ * sanlk_lockspace arg, or cleared if not set.
  */
 
 int sanlock_write_lockspace(struct sanlk_lockspace *ls, int max_hosts,
 			    uint32_t flags, uint32_t io_timeout);
 
+/*
+ * initialize one host_id lease in the lockspace
+ */
+int sanlock_init_lockspace_host(struct sanlk_lockspace *ls,
+				char *our_host_name,
+				uint64_t generation,
+				uint64_t timestamp,
+				uint32_t flags,
+				uint32_t io_timeout);
+
 /*
  * read one host's lockspace record from disk
  *
