@@ -182,10 +182,15 @@ struct host_status {
 	uint64_t owner_generation;
 	uint64_t timestamp; /* remote monotime */
 	uint64_t set_bit_time;
+	uint64_t found_dead_ext_gen;
+	uint64_t wrote_dead_ext_gen;
 	uint16_t io_timeout;
 	uint16_t lease_bad;
+	uint8_t dead_ext; /* LFL_DEAD_EXT */
+	uint8_t no_timeout; /* LFL_NO_TIMEOUT */
 	uint8_t warned_fail;
 	uint8_t warned_dead;
+	uint8_t warned_dead_ext;
 	char owner_name[NAME_ID_SIZE] __attribute__ ((nonstring));
 };
 
@@ -206,6 +211,7 @@ struct renewal_history {
 struct space {
 	struct list_head list;
 	char space_name[NAME_ID_SIZE] __attribute__ ((nonstring));
+	uint32_t version;
 	uint32_t space_id; /* used to refer to this space instance in log messages */
 	uint64_t host_id;
 	uint64_t host_generation;
@@ -218,6 +224,7 @@ struct space {
 	uint32_t rindex_op;
 	unsigned int set_max_sectors_kb;
 	int nodelay;
+	int no_timeout;
 	int sector_size;
 	int align_size;
 	int max_hosts;
@@ -388,6 +395,7 @@ struct command_line {
 	int max_worker_threads;
 	int aio_arg;
 	int write_init_io_timeout;
+	int no_timeout;
 	int set_bitmap_seconds;
 	int persistent;
 	int orphan_set;
@@ -415,6 +423,7 @@ struct command_line {
 	int max_hosts;				/* -m */
 	int res_count;
 	int sh_retries;
+	int create_delta_34;
 	uint32_t force_mode;
 	int renewal_history_size;
 	int renewal_read_extend_sec_set; /* 1 if renewal_read_extend_sec is configured */
@@ -422,6 +431,7 @@ struct command_line {
 	char our_host_name_opt[256]; /* max SANLK_NAME_LEN will be used */
 	char *file_path;
 	char *dump_path;
+	char flag_name[SANLK_NAME_LEN+1];
 	int rindex_op;
 	struct sanlk_rentry rentry;		/* -e */
 	struct sanlk_rindex rindex;		/* -x RINDEX */
@@ -438,6 +448,9 @@ int is_cmd_debug(uint32_t cmd);
 void set_cmd_debug(uint32_t cmd);
 void clear_cmd_debug(uint32_t cmd);
 
+/* main.c */
+const char *host_state_str(uint32_t flags);
+
 /* command line types and actions */
 
 #define COM_DAEMON      1
@@ -482,6 +495,9 @@ enum {
 	ACT_UPDATE,
 	ACT_REBUILD,
 	ACT_DIRECT_INIT_HOST,
+	ACT_CLIENT_INIT_HOST,
+	ACT_READ,
+	ACT_SET_HOST,
 };
 
 EXTERN int external_shutdown;
