@@ -116,7 +116,7 @@
  * multiple hosts are making requests, this flag can
  * produce unexpected results, and it would be safer
  * to read the resource, check that the current owner
- * is the one being targetted, and use that owner's
+ * is the one being targeted, and use that owner's
  * lver + 1 as the specifically requested lver.
  */
 
@@ -150,6 +150,12 @@ int sanlock_acquire(int sock, int pid, uint32_t flags, int res_count,
 		    struct sanlk_resource *res_args[],
 		    struct sanlk_options *opt_in);
 
+int sanlock_acquire2(int sock, int pid, uint32_t flags,
+		     struct sanlk_resource *res,
+		     struct sanlk_options *opt_in,
+		     struct sanlk_host *owner_host,
+		     char **owner_name);
+
 int sanlock_release(int sock, int pid, uint32_t flags, int res_count,
 		    struct sanlk_resource *res_args[]);
 
