@@ -15,6 +15,7 @@
 #define SANLK_ERROR		-201
 #define SANLK_AIO_TIMEOUT	-202
 #define SANLK_WD_ERROR		-203
+#define SANLK_VERSION_INCOMPAT	-204
 
 /* run_ballot */
 
@@ -51,6 +52,8 @@
 #define SANLK_ACQUIRE_OTHER	-245
 #define SANLK_ACQUIRE_SHRETRY	-246
 #define SANLK_ACQUIRE_OWNED_RETRY -247
+#define SANLK_ACQUIRE_OWNER_GEN_REVERTED -248
+#define SANLK_ACQUIRE_OWNED_NO_TIMEOUT -249
 
 #define SANLK_RELEASE_LVER	-250
 #define SANLK_RELEASE_OWNER	-251
@@ -60,6 +63,7 @@
 #define SANLK_RENEW_OWNER	-260
 #define SANLK_RENEW_DIFF	-261
 #define SANLK_HOSTID_BUSY	-262
+#define SANLK_RENEW_DEAD_EXT	-263
 
 /* request_token */
 
@@ -75,4 +79,6 @@
 #define SANLK_RINDEX_OFFSET	-277
 #define SANLK_RINDEX_DIFF	-278
 
+#define SANLK_WRONG_GENERATION  -280
+
 #endif
