@@ -17,7 +17,6 @@ import time
 from contextlib import contextmanager
 
 import pytest
-import six
 
 import sanlock
 
@@ -49,22 +48,7 @@ FILE_NAMES = [
     (u"\u05d0", "utf-8"),
 ]
 
-LOCKSPACE_OR_RESOURCE_NAMES = [
-    # Bytes are supported with python 2 and 3.
-    pytest.param(b"\xd7\x90"),
-    # Python 2 also supports str.
-    pytest.param(
-        "\xd7\x90",
-        marks=pytest.mark.skipif(
-            six.PY3,
-            reason="python 3 supports only bytes")),
-    # Python 2 also supports unicode with ascii content.
-    pytest.param(
-        u"ascii",
-        marks=pytest.mark.skipif(
-            six.PY3,
-            reason="python 3 supports only bytes")),
-]
+NAME = b"\xd7\x90"  # Unicode Hebrew Alef using utf-8 encoding.
 
 
 @pytest.mark.parametrize("filename, encoding", FILE_NAMES)
@@ -354,14 +338,14 @@ def test_add_rem_lockspace(tmpdir, sanlock_daemon, size, offset):
 
     sanlock.write_lockspace(b"ls_name", path, offset=offset, iotimeout=1)
 
-    # Since the lockspace is not acquired, we exepect to get False.
+    # Since the lockspace is not acquired, we expect to get False.
     acquired = sanlock.inq_lockspace(
         b"ls_name", 1, path, offset=offset, wait=False)
     assert acquired is False
 
     sanlock.add_lockspace(b"ls_name", 1, path, offset=offset, iotimeout=1)
 
-    # Once the lockspace is acquired, we exepect to get True.
+    # Once the lockspace is acquired, we expect to get True.
     acquired = sanlock.inq_lockspace(
         b"ls_name", 1, path, offset=offset, wait=False)
     assert acquired is True
@@ -377,7 +361,7 @@ def test_add_rem_lockspace(tmpdir, sanlock_daemon, size, offset):
 
     sanlock.rem_lockspace(b"ls_name", 1, path, offset=offset)
 
-    # Once the lockspace is released, we exepect to get False.
+    # Once the lockspace is released, we expect to get False.
     acquired = sanlock.inq_lockspace(
         b"ls_name", 1, path, offset=offset, wait=False)
     assert acquired is False
@@ -397,12 +381,12 @@ def test_add_rem_lockspace_async(tmpdir, sanlock_daemon):
     # This will take 3 seconds.
     sanlock.add_lockspace(b"ls_name", 1, path, iotimeout=1, wait=False)
 
-    # While the lockspace is being aquired, we expect to get None.
+    # While the lockspace is being acquired, we expect to get None.
     time.sleep(1)
     acquired = sanlock.inq_lockspace(b"ls_name", 1, path, wait=False)
     assert acquired is None
 
-    # Once the lockspace is acquired, we exepect to get True.
+    # Once the lockspace is acquired, we expect to get True.
     acquired = sanlock.inq_lockspace(b"ls_name", 1, path, wait=True)
     assert acquired is True
 
@@ -495,6 +479,90 @@ def test_acquire_release_resource(tmpdir, sanlock_daemon, size, offset):
     assert owners == []
 
 
+@pytest.mark.parametrize("res_name", [
+    "ascii",
+    "\u05d0",  # Hebrew Alef
+])
+def test_inquire(tmpdir, sanlock_daemon, res_name):
+    ls_path = str(tmpdir.join("ls_name"))
+    util.create_file(ls_path, MiB)
+
+    res_path = str(tmpdir.join(res_name))
+    util.create_file(res_path, 10 * MiB)
+
+    fd = sanlock.register()
+
+    # No lockspace yet.
+    assert sanlock.inquire(slkfd=fd) == []
+
+    sanlock.write_lockspace(b"ls_name", ls_path, offset=0, iotimeout=1)
+    sanlock.add_lockspace(b"ls_name", 1, ls_path, offset=0, iotimeout=1)
+
+    # No resources created yet.
+    assert sanlock.inquire(slkfd=fd) == []
+
+    resources = [
+        # name, offset, acquire
+        (b"res-0", 0 * MiB, True),
+        (b"res-1", 1 * MiB, False),
+        (b"res-2", 2 * MiB, True),
+        (b"res-8", 8 * MiB, False),
+        (b"res-9", 9 * MiB, True),
+    ]
+
+    for res_name, res_offset, acquire in resources:
+        sanlock.write_resource(b"ls_name", res_name, [(res_path, res_offset)])
+
+    # No resource acquired yet.
+    assert sanlock.inquire(slkfd=fd) == []
+
+    # Acquire resources.
+    for res_name, res_offset, acquire in resources:
+        if acquire:
+            sanlock.acquire(
+                b"ls_name", res_name, [(res_path, res_offset)], slkfd=fd)
+
+    time.sleep(1)
+
+    expected = [
+        {
+            "lockspace": b"ls_name",
+            "resource": b"res-0",
+            "flags": sanlock.RES_LVER,
+            "version": 1,
+            "disks": [(res_path, 0 * MiB)],
+        },
+        {
+            "lockspace": b"ls_name",
+            "resource": b"res-2",
+            "flags": sanlock.RES_LVER,
+            "version": 1,
+            "disks": [(res_path, 2 * MiB)],
+        },
+        {
+            "lockspace": b"ls_name",
+            "resource": b"res-9",
+            "flags": sanlock.RES_LVER,
+            "version": 1,
+            "disks": [(res_path, 9 * MiB)],
+        },
+    ]
+
+    # Check acquired resources using snlkfd.
+    assert sanlock.inquire(slkfd=fd) == expected
+
+    # Check acquired resources using pid.
+    assert sanlock.inquire(pid=os.getpid()) == expected
+
+    for res_name, res_offset, acquire in resources:
+        if acquire:
+            sanlock.release(
+                b"ls_name", res_name, [(res_path, res_offset)], slkfd=fd)
+
+    # All resource released.
+    assert sanlock.inquire(slkfd=fd) == []
+
+
 @pytest.mark.parametrize("align, sector", [
     # Invalid alignment
     (KiB, sanlock.SECTOR_SIZE[0]),
@@ -563,42 +631,36 @@ def raises_sanlock_errno(expected_errno=errno.ECONNREFUSED):
     assert e.value.errno == expected_errno
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_rem_lockspace_parse_args(no_sanlock_daemon, name, filename, encoding):
+def test_rem_lockspace_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     with raises_sanlock_errno():
-        sanlock.rem_lockspace(name, 1, path, 0, wait=False)
+        sanlock.rem_lockspace(NAME, 1, path, 0, wait=False)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_add_lockspace_parse_args(no_sanlock_daemon, name, filename, encoding):
+def test_add_lockspace_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     with raises_sanlock_errno():
-        sanlock.add_lockspace(name, 1, path, 0, wait=False)
+        sanlock.add_lockspace(NAME, 1, path, 0, wait=False)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_write_lockspace_parse_args(
-        no_sanlock_daemon, name, filename, encoding):
+def test_write_lockspace_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     with raises_sanlock_errno():
-        sanlock.write_lockspace(name, path)
+        sanlock.write_lockspace(NAME, path)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_write_resource_parse_args(
-        no_sanlock_daemon, name, filename, encoding):
+def test_write_resource_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     disks = [(path, 0)]
     with raises_sanlock_errno():
-        sanlock.write_resource(name, b"res_name", disks)
+        sanlock.write_resource(NAME, b"res_name", disks)
 
     with raises_sanlock_errno():
-        sanlock.write_resource(b"ls_name", name, disks)
+        sanlock.write_resource(b"ls_name", NAME, disks)
 
 
 def test_write_resource_path_length(no_sanlock_daemon):
@@ -611,17 +673,15 @@ def test_write_resource_path_length(no_sanlock_daemon):
         sanlock.write_resource(b"ls_name", b"res_name", [(path, 0)])
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_release_resource_parse_args(
-        no_sanlock_daemon, name, filename, encoding):
+def test_release_resource_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     disks = [(path, 0)]
     with raises_sanlock_errno():
-        sanlock.release(name, b"res_name", disks)
+        sanlock.release(NAME, b"res_name", disks)
 
     with raises_sanlock_errno():
-        sanlock.release(b"ls_name", name, disks)
+        sanlock.release(b"ls_name", NAME, disks)
 
 
 def test_release_resource_path_length(no_sanlock_daemon):
@@ -634,17 +694,16 @@ def test_release_resource_path_length(no_sanlock_daemon):
         sanlock.release(b"ls_name", b"res_name", [(path, 0)])
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
 def test_read_resource_owners_parse_args(
-        no_sanlock_daemon, name, filename, encoding):
+        no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     disks = [(path, 0)]
     with raises_sanlock_errno():
-        sanlock.read_resource_owners(name, b"res_name", disks)
+        sanlock.read_resource_owners(NAME, b"res_name", disks)
 
     with raises_sanlock_errno():
-        sanlock.read_resource_owners(b"ls_name", name, disks)
+        sanlock.read_resource_owners(b"ls_name", NAME, disks)
 
 
 def test_read_resource_owners_path_length(no_sanlock_daemon):
@@ -657,67 +716,31 @@ def test_read_resource_owners_path_length(no_sanlock_daemon):
         sanlock.read_resource_owners(b"ls_name", b"res_name", [(path, 0)])
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
-def test_get_hosts_parse_args(no_sanlock_daemon, name):
+def test_get_hosts_parse_args(no_sanlock_daemon):
     with raises_sanlock_errno():
-        sanlock.get_hosts(name, 1)
+        sanlock.get_hosts(NAME, 1)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_inq_lockspace_parse_args(no_sanlock_daemon, name, filename, encoding):
+def test_inq_lockspace_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     with raises_sanlock_errno():
-        sanlock.inq_lockspace(name, 1, path, wait=False)
+        sanlock.inq_lockspace(NAME, 1, path, wait=False)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
-def test_reg_event_parse_args(no_sanlock_daemon, name):
+def test_reg_event_parse_args(no_sanlock_daemon):
     with raises_sanlock_errno():
-        sanlock.reg_event(name)
+        sanlock.reg_event(NAME)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
-def test_end_event_parse_args(no_sanlock_daemon, name):
+def test_end_event_parse_args(no_sanlock_daemon):
     with raises_sanlock_errno(errno.EALREADY):
-        sanlock.end_event(-1, name)
+        sanlock.end_event(-1, NAME)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
-def test_set_event_parse_args(no_sanlock_daemon, name):
+def test_set_event_parse_args(no_sanlock_daemon):
     with raises_sanlock_errno():
-        sanlock.set_event(name, 1, 1, 1)
-
-
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
-@pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_init_lockspace_parse_args(
-        no_sanlock_daemon, name, filename, encoding):
-    path = util.generate_path("/tmp/", filename, encoding)
-    with raises_sanlock_errno(errno.ENODEV):
-        sanlock.init_lockspace(name, path)
-
-
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
-@pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_init_resource_parse_args(no_sanlock_daemon, name, filename, encoding):
-    path = util.generate_path("/tmp/", filename, encoding)
-    disks = [(path, 0)]
-    with raises_sanlock_errno(errno.ENOENT):
-        sanlock.init_resource(b"ls_name", name, disks)
-    with raises_sanlock_errno(errno.ENOENT):
-        sanlock.init_resource(name, b"res_name", disks)
-
-
-def test_init_resource_path_length(no_sanlock_daemon):
-    path = "x" * constants.SANLK_PATH_LEN
-    with pytest.raises(ValueError):
-        sanlock.init_resource(b"ls_name", b"res_name", [(path, 0)])
-
-    # init_resource access storage directly.
-    path = "x" * (constants.SANLK_PATH_LEN - 1)
-    with raises_sanlock_errno(errno.ENAMETOOLONG):
-        sanlock.init_resource(b"ls_name", b"res_name", [(path, 0)])
+        sanlock.set_event(NAME, 1, 1, 1)
 
 
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
@@ -751,17 +774,16 @@ def test_read_resource_path_length(no_sanlock_daemon):
         sanlock.read_resource(path)
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_request_parse_args(no_sanlock_daemon, name, filename, encoding):
+def test_request_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     disks = [(path, 0)]
 
     with raises_sanlock_errno():
-        sanlock.request(b"ls_name", name, disks)
+        sanlock.request(b"ls_name", NAME, disks)
 
     with raises_sanlock_errno():
-        sanlock.request(name, b"res_name", disks)
+        sanlock.request(NAME, b"res_name", disks)
 
 
 def test_request_path_length(no_sanlock_daemon):
@@ -774,17 +796,16 @@ def test_request_path_length(no_sanlock_daemon):
         sanlock.request(b"ls_name", b"res_name", [(path, 0)])
 
 
-@pytest.mark.parametrize("name", LOCKSPACE_OR_RESOURCE_NAMES)
 @pytest.mark.parametrize("filename,encoding", FILE_NAMES)
-def test_acquire_parse_args(no_sanlock_daemon, name, filename, encoding):
+def test_acquire_parse_args(no_sanlock_daemon, filename, encoding):
     path = util.generate_path("/tmp/", filename, encoding)
     disks = [(path, 0)]
 
     with raises_sanlock_errno():
-        sanlock.acquire(b"ls_name", name, disks, pid=os.getpid())
+        sanlock.acquire(b"ls_name", NAME, disks, pid=os.getpid())
 
     with raises_sanlock_errno():
-        sanlock.acquire(name, b"res_name", disks, pid=os.getpid())
+        sanlock.acquire(NAME, b"res_name", disks, pid=os.getpid())
 
 
 def test_acquire_path_length(no_sanlock_daemon):
@@ -795,3 +816,63 @@ def test_acquire_path_length(no_sanlock_daemon):
     path = "x" * (constants.SANLK_PATH_LEN - 1)
     with raises_sanlock_errno():
         sanlock.acquire(b"ls_name", b"res_name", [(path, 0)], pid=os.getpid())
+
+
+def test_lvb(tmpdir, sanlock_daemon):
+    ls_path = str(tmpdir.join("ls_name"))
+    util.create_file(ls_path, MiB)
+    res_path = str(tmpdir.join("res_name"))
+    util.create_file(res_path, MiB)
+    sanlock.write_lockspace(b"ls_name", ls_path, offset=0, iotimeout=1)
+    sanlock.add_lockspace(b"ls_name", 1, ls_path, offset=0, iotimeout=1)
+    disks = [(res_path, 0)]
+    sanlock.write_resource(b"ls_name", b"res_name", disks)
+
+    fd = sanlock.register()
+
+    # Test reading unset lvb.
+
+    sanlock.acquire(b"ls_name", b"res_name", disks, slkfd=fd, lvb=True)
+    result = sanlock.get_lvb(b"ls_name", b"res_name", disks, 512)
+    sanlock.release(b"ls_name", b"res_name", disks, slkfd=fd)
+
+    assert result == b"\0" * 512
+
+    lvb_data = b"first\0second"
+    lvb_sector = lvb_data.ljust(512, b"\0")
+
+    sanlock.acquire(b"ls_name", b"res_name", disks, slkfd=fd, lvb=True)
+    sanlock.set_lvb(b"ls_name", b"res_name", disks, lvb_sector)
+    sanlock.release(b"ls_name", b"res_name", disks, slkfd=fd)
+
+    # Test reading complete sector (typical usage).
+
+    sanlock.acquire(b"ls_name", b"res_name", disks, slkfd=fd, lvb=True)
+    result = sanlock.get_lvb(b"ls_name", b"res_name", disks, len(lvb_sector))
+    sanlock.release(b"ls_name", b"res_name", disks, slkfd=fd)
+
+    assert result == lvb_sector
+
+    # Test reading less than sector size.
+
+    sanlock.acquire(b"ls_name", b"res_name", disks, slkfd=fd, lvb=True)
+    result = sanlock.get_lvb(b"ls_name", b"res_name", disks, len(lvb_data))
+    sanlock.release(b"ls_name", b"res_name", disks, slkfd=fd)
+
+    assert result == lvb_data
+
+    # Test failure to write more than sector size.
+
+    sanlock.acquire(b"ls_name", b"res_name", disks, slkfd=fd, lvb=True)
+    with raises_sanlock_errno(errno.E2BIG):
+        sanlock.set_lvb(b"ls_name", b"res_name", disks, lvb_sector + b"x")
+
+
+def test_lvb_invalid_value():
+    disks = [("/no/such/path", 0)]
+
+    with pytest.raises(ValueError):
+        sanlock.get_lvb(b"ls_name", b"res_name", disks, 0)
+
+    with pytest.raises(ValueError):
+        sanlock.get_lvb(b"ls_name", b"res_name", disks, 4097)
